from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *

from Acquisition import aq_parent, aq_base, aq_inner

from config import *
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import createExprContext

from ContextCriteriaMixin import ContextCriteriaMixin
from interfaces import IContextAction

import schemata

class ContextAction(BaseContent, ContextCriteriaMixin):
    """
    A ContextAction represents a context-dependent action. The
    criteria found in the mixin class ContextCriteriaMixin are used
    by ContextActionsTool to determine whether or not a given action
    should be offered in the current context.
    """
    
    __implements__  = BaseContent.__implements__ + \
                      ContextCriteriaMixin.__implements__ + \
                      (IContextAction,)
    
    security        = ClassSecurityInfo()
    schema          = schemata.ContextActionSchema
    
    portal_type     = 'ContextAction'
    meta_type       = 'ContextAction' 
    archetype_name  = 'Action' 

    allow_discussion = 0 
    global_allow     = 0
    allowed_content_types = []
    
    security.declareProtected(CMFCorePermissions.View, 'evaluateTarget')
    def evaluateTarget(self, context):
        """Evaluate the action target TALES expression in the given context.
        Note that this is done automatically on the values returned from the
        portal_contextactions tool's getActions() method, so it is only 
        useful in cases where you have an instance of a ContextAction and need
        to resolve its target to a real URL.
        """
        
        portal = getToolByName(self, 'portal_url').getPortalObject()
        
        # Find folder (code courtesy of CMFCore.ActionsTool)
        if context is None or not hasattr(context, 'aq_base'):
            folder = portal
        else:
            folder = context
            # Search up the containment hierarchy until we find an
            # object that claims it's a folder.
            while folder is not None:
                if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                    # found it.
                    break
                else:
                    folder = aq_parent(aq_inner(folder))
    
        __traceback_info__ = (folder, portal, context, \
                                 self.getTargetExpression())

        ec = createExprContext(folder, portal, context)
        return Expression(self.getTargetExpression())(ec)
          
    # Implement "abstract" method from ContextCriteriaMixin
    def getExistingCategories(self):
        """
        Get a DisplayList of categories currently in use. Used as the vocabulary
        for the categories field.
        """
        actionsTool = getToolByName(self, TOOLNAME)
        lst = DisplayList()        
        for category in actionsTool.getAvailableCategories():
            lst.add(category, category)
        return lst.sortedByValue()


registerType(ContextAction)
