from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *

from Products.CMFCore import CMFCorePermissions

from config import *
from interfaces import IContextActionManager

class ContextActionManager:
    """
    Mixin class to provide a cleaner API for ContextActionBundle and 
    ContextActionFolder to manage folders and actions. The aim is to
    make these seems less content-ish and more like a structured class
    hierarchy.
    """
    
    __implements__ = (IContextActionManager,)
    
    security = ClassSecurityInfo()
    
    #
    # Contained actions management
    #
    
    security.declareProtected(CMFCorePermissions.View, 'getAction')
    def getAction(self, id):
        """Get the ContextAction with the specified id. Raises an AttributeError
        if the action cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Action %s does not exist" % (id,)
        return getattr(self, id)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllActions')
    def getAllActions(self):
        """Get a list of all ContextActions managed directly by this container
        """
        return self.objectValues(['ContextAction'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                             'addAction')
    def addAction(self, id, title, description, categories, targetExpression,
                    contentTypes = None, matchPath = None, 
                    states = None, templateIds = None, matchRoles = None,
                    expression = None, active = 1):
        """Add a new ContextAction. All arguments without default values are
        required. Categories should be a list, as should contentTypes,
        states, templateIds and matchRoles if given. Raises an AttributeError
        if the id is already in use. Returns the newly created action.
        """
        if id in self.objectIds():
            raise AttributeError, "Action %s already exists" % (id,)
        
        self.invokeFactory('ContextAction', id)
        action = getattr(self, id)
        
        action.setTitle(title)
        action.setDescription(description)
        action.setCategories(categories)
        action.setTargetExpression(targetExpression)
        
        if contentTypes:
            action.setContentTypes(contentTypes)
        if matchPath:
            action.setMatchPath(matchPath)
        if states:
            action.setStates(states)
        if templateIds:
            action.setTemplateIds(templateIds)
        if matchRoles:
            action.setMatchRoles(matchRoles)
        if expression:
            action.setExpression(expression)
        
        # No condition - we want true or false!
        action.setActive(active)
            
        return action
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'removeAction')
    def removeAction(self, id):
        """Remove the given ContextAction. Rasises an AttributeError if it
        cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Action %s does not exist" % (id,)
        self.manage_delObjects([id])

    #
    # Contained action folders management
    #
    
    security.declareProtected(CMFCorePermissions.View, 'getActionFolder')
    def getActionFolder(self, id):
        """Get the ContextActionFolder with the given id. Raises an
        AttributeError if it cannot be found.
        """
        if not id in self.objectIds():
            raise AttributeError, "Action folder %s does not exist" % (id,)
        return getattr(self, id)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllActionFolders')
    def getAllActionFolders(self):
        """Get a list of all ActionFolders managed directly by this container.
        """
        return self.objectValues(['ContextActionFolder'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'addActionFolder')
    def addActionFolder(self, id, title, description=None):
        """Add a new ContextActionFolder. Raises an AttributeError if the id is
        already in use. Returns the newly created action folder.
        """
        if id in self.objectIds():
            raise AttributeError, "Action folder %s already exists" % (id,)
        
        self.invokeFactory('ContextActionFolder', id)
        folder = getattr(self, id)
        
        folder.setTitle(title)
        
        if description:
            folder.setDescription(description)
        
        return folder
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'removeActionFolder')
    def removeActionFolder(self, id):
        """Remove the given ContextActionFolder. Raises an AttributeError if it
        cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Action folder %s does not exist" % (id,)
        self.manage_delObjects([id])
        
