from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.Archetypes.public import *

from Products.CMFCore import CMFCorePermissions

from ContextCriteriaMatcher import ContextCriteriaMatcher
from config import *
from interfaces import IContextActionsTool

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import createExprContext

from Acquisition import aq_base, aq_parent , aq_inner
from Acquisition import Implicit
from Globals import InitializeClass
from AccessControl import getSecurityManager

import schemata
    
class ActionWrapper(Implicit):
    """A decorator for ContextAction, which provides the evaluated
    target rather than the original TALES expression.
    """
    
    security = ClassSecurityInfo()
    security.declareObjectPublic ()
    
    def __init__(self, brain, target):
        self._brain = brain
        self._target = target

    security.declarePublic('getId')
    def getId(self):
        """The id of the action object"""
        return self._brain.getId

    security.declarePublic('Title')
    def Title(self):
        """The title of the action object"""
        return self._brain.Title
        
    security.declarePublic('Description')
    def Description(self):
        """The description of the action object"""
        return self._brain.Description

    security.declarePublic('getCategories')
    def getCategories(self):
        """A tuple of the categories the action is in"""
        return self._brain.getCategories

    security.declarePublic('getContentTypes')
    def getContentTypes(self):
        """A tuple of the content types the action is relevant to"""
        return self._brain.getContentTypes

    security.declarePublic('getMatchPath')
    def getMatchPath(self):
        """The path the action is relevant to"""
        return self._brain.getMatchPath

    security.declarePublic('getStates')
    def getStates(self):
        """A tuple of the workflow states the action is relevant to"""
        return self._brain.getStates
    
    security.declarePublic('getTemplateIds')
    def getTemplateIds(self):
        """A tuple of the template ids the action is relevant to"""
        return self._brain.getTemplateIds

    security.declarePublic('getMatchRoles')
    def getMatchRoles(self):
        """A tuple of the portal roles the action is relevant to"""
        return self._brain.getMatchRoles
        
    security.declarePublic('getTarget')
    def getTarget(self):
        """The (evaluated) target URL of the action"""
        return self._target
        
    security.declarePublic('getActionURL')
    def getActionURL(self):
        """The URL to the action object itself, inside the tool"""
        return self._brain.getURL()

InitializeClass(ActionWrapper)

moduleSecurity = ModuleSecurityInfo ()
moduleSecurity.declarePublic ('ActionWrapper')
    
class ContextActionsTool(UniqueObject, OrderedBaseFolder, 
                            ContextCriteriaMatcher):
    """
    Defines the tool portal_contextactions, which stores
    ContextActions with context-specific action definitions in the
    portal. ContextActions are stored in ContextActionBundles, which
    may be organised internally with ContextActionFolders. Action
    lookups are catalog-based, so the complexity of this structure
    is arbitrary. 
    
    The methods of the IContextActionTool interface should be used for 
    managing bundles, rather than the usual invokeFactory() and friends,
    for clarity and consistency.
    """
    
    __implements__ = OrderedBaseFolder.__implements__ + \
                        ContextCriteriaMatcher.__implements__ + \
                        (IContextActionsTool,)
    
    schema         = OrderedBaseFolderSchema + \
                        schemata.ContextActionsToolSchema
    security       = ClassSecurityInfo()

    portal_type    = 'ContextActionsTool' 
    meta_type      = 'ContextActionsTool' 
    archetype_name = 'Contextual actions tool'
    content_icon   = 'contextactionstool.gif'
    
    allowed_content_types = ['ContextActionBundle']
    global_allow          = 1
    #filter_content_types  = 1
    allow_discussion      = 0
    
    typeDescription = "The portal_contextactions tool manages actions which " \
                      "are made available depending on the current context " \
                      "in the portal."
    typeDescMsgId   = 'description_edit_contextactionstool'
        
    # tool-constructors have no id argument, the id is fixed
    def __init__(self):
        OrderedBaseFolder.__init__(self,TOOLNAME)
        ContextCriteriaMatcher.__init__(self)
        self._initIndexes()

    #
    # Action queries
    #

    security.declarePublic('getActions')
    def getActions(self, context, templateId=None, categories=None, **kwargs):
        """Get the sum of actions to be displayed for a given context. You must
        also pass in the current template id, or None if it is not relevant,
        and the categories to consider, or None if all categories should be
        considered.
        
        Additional query parameters can be passed via keyword arguments.
        Returns a catalog query result set.
        """

        # Inject template id and categories into the kwargs sent to the query
        if templateId:
            kwargs['match_template_ids'] = templateId
         
        if 'match_categories' in kwargs:
            del kwargs['match_categories']   
        if categories:
            kwargs['match_categories'] = categories
            
        matches = self.match(context, **kwargs)            
        return self.wrapActions(context, matches)
        
    def wrapActions(self, context, matches):
        """Given a list of matches (a catalog result set), return a list
        
        """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        
        if context is None or not hasattr(context, 'aq_base'):
            folder = portal
        else:
            folder = context
            while folder is not None:
                if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                    break
                else:
                    folder = aq_parent(aq_inner(folder))
        
        ec = createExprContext(folder, portal, context)
        actions = []
        for brain in matches:
            expression = brain.getTargetExpression
            __traceback_info__ = (folder, portal, context, expression)
            if expression:
                t = Expression(expression)(ec)
            else:
                t = ''
            wrapper = ActionWrapper(brain, t)
            actions.append(wrapper.__of__(context))
        
        return actions


    # filterContextQuery template method - declared private in base class
    def filterContextQuery(self, query):
        
        # If we are to disable any categories, get the list of categories to
        # consider (either all or the ones passed in with the categories 
        # parameter) and subtract the ones to diable.
        
        categories = query.get('match_categories', None)
        disabledCategories = self.getDisabledCategories()
        
        if disabledCategories:
        
            allCategories = []  
            if categories:
                allCategories = categories
            else:
                allCategories = self.getAvailableCategories()
            
            filteredCategories = filter(lambda x: x not in disabledCategories, 
                                          allCategories)

            # If categories becomes empty, it should match nothing
            # (which is different from the case when no categories are disabled
            # and categories is [] - in this case, we should match all!), we
            # need to match nothing. Thus, short circuit here
            
            if not filteredCategories:
                query['match_categories'] = (MATCHALL,)
            else:
                query['match_categories'] = filteredCategories
        
        # Similarly, if we are to disable some roles, find the user's current
        # roles and subtract the disabled ones.
        
        disabledRoles = self.getDisabledRoles()
        
        if disabledRoles:
        
            allRoles = []
            if 'match_user_roles' in query:
                allRoles = query['match_user_roles']
            else:
                allRoles = \
                    getSecurityManager().getUser().getRolesInContext(context)
        
            filteredRoles = filter(lambda x: x not in disabledRoles, allRoles)

            query['match_user_roles'] = filteredRoles
        
        return query


    security.declareProtected(CMFCorePermissions.View, 'evaluateExpression')
    def evaluateExpression(self, expression, context):
        """Evaluate the action target TALES expression (found a result record
        from a match). Note that the objects returned by getActions() contain
        evaluated targets, so this method is usually not needed.
        """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        # Find folder (code courtesy of CMFCore.ActionsTool)
        if context is None or not hasattr(context, 'aq_base'):
            folder = portal
        else:
            folder = context
            # Search up the containment hierarchy until we find an
            # object that claims it's a folder.
            while folder is not None:
                if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                    # found it.
                    break
                else:
                    folder = aq_parent(aq_inner(folder))

        __traceback_info__ = (folder, portal, context, expression)

        ec = createExprContext(folder, portal, context)
        return Expression(expression)(ec)


    #
    # Bundle management API
    #
            
    security.declareProtected(CMFCorePermissions.View, 'getBundle')
    def getBundle(self, providerName):
        """Get the bundle for the given provider. Raises an AttributeError if
        the bundle cannot be found.
        """
        if providerName not in self.objectIds():
            raise AttributeError, "Bundle %s does not exist" % (providerName,)
        return getattr(self, providerName)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllBundles')
    def getAllBundles(self):
        """Get a list of all bundles managed by the tool.
        """
        return self.objectValues(['ContextActionBundle'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'addBundle')
    def addBundle(self, providerName, title, description = None):
        """Add a new action bundle with the given title and description.
        The id of the bundle will be the same as providerName, which should
        be set to the name of the product which is providing the action bundle.
        Raises an AttributeError if a bundle for this provider already exists.
        Returns the newly created bundle.
        """
        if providerName in self.objectIds():
            raise AttributeError, "Bundle %s already exists" % (providerName,)
        self.invokeFactory('ContextActionBundle', providerName)
        bundle = getattr(self, providerName)
        bundle.setTitle(title)
        if description:
            bundle.setDescription(description)
        return bundle
        
    security.declareProtected(CMFCorePermissions.View, 'removeBundle')
    def removeBundle(self, providerName):
        """Remove the bundle managed by the given provider. Raises an
        AttributeError if it cannot be found.
        """
        if not providerName in self.objectIds():
            raise AttributeError, "Bundle %s does not exist" % (providerName,)
        self.manage_delObjects([providerName])


    #
    # Initalisation
    #

    security.declarePrivate('enumerateIndexes')
    def enumerateIndexes(self):
        """Enumerate the indexes being used by this tool, which will be
        registered in the internal catalog when the tool is first created.
        """
        return ContextCriteriaMatcher.enumerateIndexes(self) + ()
        
    security.declarePrivate('enumerateColumns')
    def enumerateColumns(self):
        """Enumerate the metadata columns being used by this tool, which will be 
        registered in the internal catalog when the tool is first created.
        """
        return ContextCriteriaMatcher.enumerateColumns(self) + (
            'getTargetExpression'
           , )
 
    #
    # Vocabulary methods
    #
 
    security.declarePrivate('getAvailableRoles')
    def getAvailableRoles(self):
        """
        Get a list of the roles currently in use in the portal. This helper
        method aids defining vocabularies for subclass tools which allow the 
        user to select which roles to enable.
        """
        membershipTool = getToolByName(self, 'portal_membership')
        l = DisplayList()
        for role in membershipTool.getPortalRoles():
            l.add(role, role)
        return l        
 
registerType(ContextActionsTool)

