from AccessControl import ClassSecurityInfo
from interfaces import IContextCriteriaMixin

from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.Archetypes.public import *

import schemata

class ContextCriteriaMixin:
    """
    Mixin class providing the fields which allow contextual items to be
    matched to a context. These fields are indexed in the special catalog.
    """
        
    __implements__ = (IContextCriteriaMixin,)
    
    schema = schemata.ContextCriteriaMixinSchema
    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                                'getAvailableContentTypes')
    def getAvailableContentTypes(self):
        """
        Get a DisplayList of available content types. Used as the
        vocabulary for the contentTypes field.
        """       
        typesTool = getToolByName(self, 'portal_types')
        names = DisplayList()
        ptypes = typesTool.listContentTypes()
        for name in ptypes:
            names.add(name, typesTool.getTypeInfo(name).Title())
        return names.sortedByValue()


    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                                'getAvailableRoles')
    def getAvailableRoles(self):
        """
        Get a DisplayList of available roles. Used as the vocabulary for
        the roles field.
        """
        membershipTool = getToolByName(self, 'portal_membership')
        lst = DisplayList()
        for role in membershipTool.getPortalRoles():
            lst.add(role, role) 
        return lst.sortedByValue()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'getAvailableWorkflowStates')
    def getAvailableWorkflowStates(self):
        """
        Get a list of available workflow states.
        """
        workflowTool = getToolByName(self, 'portal_workflow')
        uniqueStates = {}
        # Find each unique state
        for wfName in workflowTool.listWorkflows():
            wf = getattr(workflowTool, wfName)
            for (name, obj) in wf.states.items():
                uniqueStates[name] = name
        # Convert to a DisplayList()
        l = DisplayList()
        for (name, title) in uniqueStates.items():
            l.add(name, title)
        
        return l.sortedByValue()
        

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                             'getExistingCategories')
    def getExistingCategories(self):
        """
        Get a DisplayList of categories currently in use. Used as the vocabulary
        for the categories field.
        
        **You must implement this in the class using this mixin, since the 
        value cannot be determined by the mixin**
        """
        # NOTE: This shouldn't be here. :-) No way to find out without access
        # to the tool, and in this mixin, we don't know what the tool is called.
        # Leave undefined and require subclasses to implement.
        raise AttributeError, "getExistingCategory() not implemented " \
                              "in class deriving from ContextCriteriaMixin"
                              