from Interface import Base

class IContextAction(Base):
    """
    Interface for contextual actions. These are indexed in a special catalogue
    and obtained via a catalogue search which matches actions to a given
    context. The interface IContextCriteriaMatcher defines the matcher part
    of this catalogue. The interface IContextCriteriaMixin defines the objects
    this catalogue indexes - a class implementing IContextAction should also
    implement IContextCriteriaMixin (and derive from the ContextCriteriaMixin
    class).
    """
    
    def getTargetExpression():
        """An action resolves to a TALES expression, obtained via this method.
        This expression will be evaluated in the current context to obtain a
        URL which will be called if the user invokes the action.
        """
        
        pass

    def evaluateTarget(context):
        """Evaluate the action target TALES expression in the given context.
        Note that this is done automatically on the values returned from the
        portal_contextactions tool's getActions() method, so it is only 
        useful in cases where you have an instance of a ContextAction and need
        to resolve its target to a real URL.
        """
        
        pass


