from Interface import Base

class IContextActionsTool(Base):
    """
    Interface for context-dependent action tools. A context-dependent action 
    tool contains IContextAction-implementing objects, and can find the actions
    relevant in a given context via a call to getActions ().
    """

    def getActions(context, tempateId, categories, **kwargs):
        """Return a catalog query search result set consisting of the
        actions relevant to the given context, and/or the given template id.
        If the template id is not relevant, set it to None. 
        
        Return only actions in the given categories (a list). Set categories
        to None or [] if all categories should be considered.
        
        Note that the list of disabled roles and categories set on the
        tool (via the configlet or the schema mutators directly) may
        impact on the matched categories and roles.
        
        You may pass in additional parameters to match other catalog indexes.
        """
        
        pass
    
    #
    # Bundle management API
    #
                
    def getBundle(providerName):
        """Get the bundle for the given provider. Raises an AttributeError if
        the bundle cannot be found.
        """
        pass
        
    def getAllBundles():
        """Get a list of all bundles managed by the tool.
        """
        pass
   
    def addBundle(providerName, title, description=None):
        """Add a new action bundle with the given title and description.
        The id of the bundle will be the same as providerName, which should
        be set to the name of the product which is providing the action bundle.
        Raises an AttributeError if a bundle for this provider already exists.
        Returns the newly created bundle.
        """
        pass
        
    def removeBundle(providerName):
        """Remove the bundle managed by the given provider. Raises an
        AttributeError if it cannot be found.
        """
        pass    

# end of class IContextActionsTool

