from Interface import Base

class IContextCriteriaMatcher(Base):
    """
    Interface for classes which match criteria set in a
    ContextCriteriaMixin.
    """

    #Methods

    def match(context, **kw):
        """
        Get a catalog search result of all active items matching the
        given context's path, portal type, the current user's role,
        the currently enabled categories and other criteria found as
        indexes in the catalog. All the accessors of the non-trivial
        fields in the schema of ContextCriteriaMixin are indexed and
        may be matched. 
        
        Note that the templateId field must be passed in as a keyword 
        argument if it is be considered, with the name 'match_template_ids'. 
        This is because the current template id cannot be found from the 
        context object alone.
        
        If you add additional indexes to the catalog, you can use the
        kwargs to search them. Any kwarg matching an index name will be
        passed along to the query, and will override any value which would
        otherwise be automatically obtained from the context.
        
        After matching the catalog indexes, the TALES expressions set in
        the indexed items' getExpression methods are evaluated, if they
        are set. Only those items with expressions evaluating to true
        will be returned.
        
        All of this is done on the catalogue metadata fields, so the
        objects themselves are not woken up from the ZODB. The final
        result set is returned.
        """        
        pass
        
    def filterContextQuery(query):
        """Subclasses mixing in ContextCriteriaMatcher may implement this method
        to filter the query being sent to searchResults(). The 'query' param is
        a dict containing catalog indexes and keys and search criteria as 
        values. Hence, subclasses can modify this dict as appropriate to do any
        final filtering before the query is executed.
        """
        pass

    def matchByContext(context, **kw):
        """
        Does the actual matching for match() above, but does not evaluate
        any TAL expressions. Declared private, as it is not intended for
        TTW use.
        """
        
        pass
        
    def evaluateCondition(self, brain, context):
        """Evaluate a result set brain's condition to see if it should be 
        displayed.
        """
        
        pass

# end of class IContextCriteriaMatcher

