# 
# PloneContextActions schemata. Contains the schemata for the 
# ContextActionsTool, and ContextAction, with their base classes and mixins.
# 

from Products.Archetypes.public import *

from Products.AddRemoveWidget import AddRemoveWidget

# Put description on main edit page
ContextActionsBaseSchema = BaseSchema.copy()
ContextActionsBaseSchema['description'].isMetadata = False
ContextActionsBaseSchema['description'].schemata = 'default'

#
# Common schema for classes deriving from ContextCriteriaMatcher
#

ContextCriteriaMatcherSchema = Schema((
        LinesField('disabledRoles',
            enforceVocabulary=0,
            vocabulary='getAvailableRoles',
            widget=AddRemoveWidget
            (allow_add=1,
                description="For items with restrictions on roles, you can "   \
                 "disable certain roles by selecting them here. Note "         \
                 "that you may add roles not currently in the portal, in the " \
                 "event that they will be added later. Unrecognised roles "    \
                 "be ignored later.",
                description_msgid='ContextActions_help_disabledRoles',
                i18n_domain='ContextActions',
                label="Disabled roles",
                label_msgid='ContextActions_label_disabledRoles',
            ),
        ),
        
        LinesField('disabledCategories',
            enforceVocabulary=0,
            vocabulary='getAvailableCategories',
            widget=AddRemoveWidget
            (allow_add=1,
                description="Items may be organised by category. You may "     \
                 "disable certain categories here. Note that you may "         \
                 "add categories not currently found in the portal, in the "   \
                 "event that they may be added later. Unrecognised "           \
                 "categories will be ignored later.",
                description_msgid='ContextActions_help_disabledCategories',
                i18n_domain='ContextActions',
                label="Disabled categories",
                label_msgid='ContextActions_label_disabledCategories',
            ),
        ),
        
    ),)
    
#
# Schema for ContextActionsTool
#

#
# Schema for ContextCriteriaMixin - has the standard fields for 
# ContextCriteriaMatcher to match on.
#

ContextCriteriaMixinSchema = Schema((

        BooleanField('active',
            accessor='isActive',
            default=1,
            widget=BooleanWidget(description="Unchecking this box will cause " \
                        "this item to be ignored in future context-dependent " \
                        "searches.",
                        description_msgid='ContextActions_help_active',
                        i18n_domain='ContextActions',
                        label='Enable',
                        label_msgid='ContextActions_label_active',),
        ),

        LinesField('categories',
            required=0,
            enforceVocabulary=0,
            vocabulary='getExistingCategories',
            widget=AddRemoveWidget(allow_add=1,
                        description="Please select the category or "           \
                        "categories the item will appear in. Normally, there " \
                        "will only be one, but in certain circumstances you "  \
                        "may wish to select several. You can also add new "    \
                        "categories, but only do this where it is absolutely " \
                        "necessary.",
                        description_msgid='ContextActions_help_categories',
                        i18n_domain='ContextActions',
                        label='Categories',
                        label_msgid='ContextActions_label_categories',),
        ),
        
        LinesField('contentTypes',
            enforceVocabulary=0,
            vocabulary='getAvailableContentTypes',
            widget=AddRemoveWidget(description="Select the content types "     \
                        "this item is relevant for.",
                        description_msgid='ContextActions_help_contentTypes',
                        i18n_domain='ContextActions',
                        label='Content types',
                        label_msgid='ContextActions_label_contentTypes',),
        ),
        
        StringField('matchPath',
            widget=StringWidget(description="If this item is relevant only "   \
                        "to a given section of the portal, enter the path "    \
                        "to the section, relative to the portal root, below. ",
                        description_msgid='ContextActions_help_path',
                        i18n_domain='ContextActions',
                        label='Path',
                        label_msgid='ContextActions_label_path',),
        ),

        LinesField('states',
            enforceVocabulary=0,
            vocabulary='getAvailableWorkflowStates',
            widget=AddRemoveWidget(description="Only display this item if "    \
                        "the current object is in one of the given workflow "  \
                        "states",
                        description_msgid='ContextActions_help_states',
                        i18n_domain='ContextActions',
                        label='Workflow states',
                        label_msgid='ContextActions_label_states',),
        ),
        
        LinesField('templateIds',
            widget=LinesWidget(description="If this item is relevant only to " \
                        "one or more specific page templates, enter the ids "  \
                        "the templates below, one per line.",
                        description_msgid='ContextActions_help_templateIds',
                        i18n_domain='ContextActions',
                        label='Templates',
                        label_msgid='ContextActions_label_templateIds',),
        ),
        
        LinesField('matchRoles',
            enforceVocabulary=0,
            vocabulary='getAvailableRoles',
            widget=AddRemoveWidget(allow_add=1,
                    description="If this item is relevant only to one or "     \
                        "more portal roles, select the roles below. You may  " \
                        "also add roles not currently found in the portal, "   \
                        "in the event that they will be added later.",
                        description_msgid='ContextActions_help_roles',
                        i18n_domain='ContextActions',
                        label='Roles',
                        label_msgid='ContextActions_label_roles',),
        ),
        
        StringField('expression',
            widget=StringWidget(description="For more control over the "       \
                        "selection process, enter a TALES expression below. "  \
                        "This will be evaluated in addition to the other "     \
                        "criteria.  (Note that such an expression is much "    \
                        "slower than using the specific criteria above, so "   \
                        "use the above filters as much as possible first)",
                        description_msgid='ContextActions_help_expression',
                        i18n_domain='ContextActions',
                        label='Expression',
                        label_msgid='ContextActions_label_expression',),
        ),
        
       
    ))


#
# ContextAction schema
#

ContextActionSchema = ContextActionsBaseSchema + ContextCriteriaMixinSchema + \
                        Schema((

        StringField('title',
            accessor='Title',
            required=1,
            widget=StringWidget(description="The title of the action, i.e. "   \
                        "what will be displayed to the user.",
                        description_msgid='ContextActions_help_title',
                        i18n_domain='ContextActions',
                        label='Title',
                        label_msgid='ContextActions_label_title',),
        ),
        
        TextField('description',
            accessor='Description',
            widget=TextAreaWidget(description="Please enter a short "          \
                        "description of the action. This will help users "     \
                        "determine its relevancy.",
                        description_msgid='ContextActions_help_description',
                        i18n_domain='ContextActions',
                        label='Description',
                        label_msgid='ContextActions_label_description',),
        ),
        
        StringField('targetExpression',
            required=1,
            widget=StringWidget(description="A TALES expression representing " \

                        "the target of the action.",
                        description_msgid='ContextActions_help_target',
                        i18n_domain='ContextActions',
                        label='Action target',
                        label_msgid='ContextActions_label_target',),
        ),  
    ),)

#
# ContextActionsTool schema
#

# Disable the id and title fields on the edit action tab

ContextActionsToolSchema = ContextCriteriaMatcherSchema + Schema((

    StringField('id',
        mode='r',
        widget=StringWidget(
            visible={'edit' : 'invisible', 'view' : 'invisible'},
        ),
    ),
    
    ComputedField('title',
        mode='r',
        expression='"Contextual actions"',
        widget=ComputedWidget(
            visible={'edit' : 'invisible', 'view' : 'invisible'},
        ),
    ),
    
    ),)
