
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextActions.tests import catc
from Products.PloneContextActions.config import *
from Products.CMFCore.utils import getToolByName

from Acquisition import aq_base

class TestInTool(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contextactions
        self.parent = self.tool

    def testAddBundle(self):
        b1 = self.parent.addBundle('b1', 'Bundle 1')
        b2 = self.parent.addBundle('b2', 'Bundle 2', 'Second bundle')
        
        self.failUnless(b1 == getattr(self.parent, 'b1'))
        self.failUnless(b2 == getattr(self.parent, 'b2'))
        
        self.assertEqual(b1.getId(), 'b1')
        self.assertEqual(b1.Title(), 'Bundle 1')
        self.assertEqual(b1.Description(), '')
        
        self.assertEqual(b2.getId(), 'b2')
        self.assertEqual(b2.Title(), 'Bundle 2')
        self.assertEqual(b2.Description(), 'Second bundle')
        
    def testAddDuplicateBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        
        try:
            self.parent.addBundle('b1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()
    
    def testRemoveBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        self.parent.removeBundle('b1')
        
        self.failIf(getattr(aq_base(self.parent), 'b1', None))
        
    def testRemoveNonExistentBundle(self):
        try:
            self.parent.removeBundle('b1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        b1 = self.parent.getBundle('b1')
        
        self.assertEqual(b1.getId(), 'b1')
        self.assertEqual(b1.Title(), 'Bundle 1')
        
    def testGetNonExistentBundle(self):
        try:
            self.parent.getBundle('b1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllBundles(self):
        self.parent.addBundle('b1', 'Bundle 1')
        self.parent.addBundle('b2', 'Bundle 2')
        
        bs = self.parent.getAllBundles()
        ids = [b.getId() for b in bs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'b1')
        self.assertEqual(ids[1], 'b2')
        
    def testGetEmptyAllBundles(self):
        bs = self.parent.getAllBundles()
        self.assertEqual(len(bs), 0)

class TestInBundle(catc.ContextActionsTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('b1', 'Bundle 1')
        self.parent = self.bundle
    
    #
    # Action folders
    #
    
    def testAddFolder(self):
        f1 = self.parent.addActionFolder('f1', 'Folder 1')
        f2 = self.parent.addActionFolder('f2', 'Folder 2', 'Second folder')
        
        self.failUnless(f1 == getattr(self.parent, 'f1'))
        self.failUnless(f2 == getattr(self.parent, 'f2'))
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        self.assertEqual(f1.Description(), '')
        
        self.assertEqual(f2.getId(), 'f2')
        self.assertEqual(f2.Title(), 'Folder 2')
        self.assertEqual(f2.Description(), 'Second folder')
        
    def testAddDuplicateFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        
        try:
            self.parent.addActionFolder('f1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        self.parent.removeActionFolder('f1')
        
        self.failIf(getattr(aq_base(self.parent), 'f1', None))
        
    def testRemoveNonExistentFolder(self):
        try:
            self.parent.removeActionFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        f1 = self.parent.getActionFolder('f1')
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        
    def testGetNonExistentFolder(self):
        try:
            self.parent.getActionFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllFolders(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        self.parent.addActionFolder('f2', 'Folder 2')
        
        fs = self.parent.getAllActionFolders()
        ids = [f.getId() for f in fs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'f1')
        self.assertEqual(ids[1], 'f2')
        
    def testGetEmptyAllFolders(self):
        fs = self.parent.getAllActionFolders()
        self.assertEqual(len(fs), 0)
        
    #
    # Actions
    #
    
    def testAddAction(self):
        a1 = self.parent.addAction('a1', 'Action 1', 'First action', 
                                    ['cat1'], 'string:foo')
                                
        a2 = self.parent.addAction('a2', 'Action 2', 'Second action', 
                                    ['cat2', 'cat3'], 'string:bar', 
                                    ['Document', 'Image'],
                                    '/foo/bar',
                                    ['published', 'private'],
                                    ['base_edit', 'base_view'],
                                    ['Member', 'Manager'],
                                    'here/displayAction',
                                    0)
        
        self.failUnless(a1 == getattr(self.parent, 'a1'))
        self.failUnless(a2 == getattr(self.parent, 'a2'))
        
        self.assertEqual(a1.getId(), 'a1')
        self.assertEqual(a1.Title(), 'Action 1')
        self.assertEqual(a1.Description(), 'First action')
        self.assertEqual(a1.getCategories(), ('cat1',))
        self.assertEqual(a1.getTargetExpression(), 'string:foo')
        self.assertEqual(a1.getContentTypes(), ())
        self.assertEqual(a1.getMatchPath(), '')
        self.assertEqual(a1.getStates(), ())
        self.assertEqual(a1.getTemplateIds(), ())
        self.assertEqual(a1.getMatchRoles(), ())
        self.assertEqual(a1.getExpression(), '')
        self.assertEqual(a1.isActive(), 1)
        
        self.assertEqual(a2.getId(), 'a2')
        self.assertEqual(a2.Title(), 'Action 2')
        self.assertEqual(a2.Description(), 'Second action')
        self.assertEqual(a2.getCategories(), ('cat2','cat3'))
        self.assertEqual(a2.getTargetExpression(), 'string:bar')
        self.assertEqual(a2.getContentTypes(), ('Document', 'Image'))
        self.assertEqual(a2.getMatchPath(), '/foo/bar')
        self.assertEqual(a2.getStates(), ('published', 'private'))
        self.assertEqual(a2.getTemplateIds(), ('base_edit', 'base_view'))
        self.assertEqual(a2.getMatchRoles(), ('Member', 'Manager'))
        self.assertEqual(a2.getExpression(), 'here/displayAction')
        self.assertEqual(a2.isActive(), 0)
        
    def testAddDuplicateAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        
        try:
            self.parent.addAction('a1', 'Ooopsie', 'Only one action allowed', 
                                    ['cat2'], 'string:bar')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        self.parent.removeAction('a1')
        
        self.failIf(getattr(aq_base(self.parent), 'a1', None))
        
    def testRemoveNonExistentAction(self):
        try:
            self.parent.removeAction('a1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        a1 = self.parent.getAction('a1')
        
        self.assertEqual(a1.getId(), 'a1')
        self.assertEqual(a1.Title(), 'Action 1')
        self.assertEqual(a1.Description(), 'First action')
        self.assertEqual(a1.getCategories(), ('cat1',))
        self.assertEqual(a1.getTargetExpression(), 'string:foo')
        
    def testGetNonExistentAction(self):
        try:
            self.parent.getAction('a1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllActions(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        self.parent.addAction('a2', 'Action 2', 'Second action',
                                ['cat2'], 'string:bar')
        
        cas = self.parent.getAllActions()
        ids = [a.getId() for a in cas]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'a1')
        self.assertEqual(ids[1], 'a2')
        
    def testGetEmptyAllActions(self):
        cas = self.parent.getAllActions()
        self.assertEqual(len(cas), 0)
        
class TestInFolder(catc.ContextActionsTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('b1', 'Bundle 1')
        self.folder = self.bundle.addActionFolder('f1', 'Folder 1')
        self.parent = self.folder

    #
    # Action folders
    #

    def testAddFolder(self):
        f1 = self.parent.addActionFolder('f1', 'Folder 1')
        f2 = self.parent.addActionFolder('f2', 'Folder 2', 'Second folder')
        
        self.failUnless(f1 == getattr(self.parent, 'f1'))
        self.failUnless(f2 == getattr(self.parent, 'f2'))
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        self.assertEqual(f1.Description(), '')
        
        self.assertEqual(f2.getId(), 'f2')
        self.assertEqual(f2.Title(), 'Folder 2')
        self.assertEqual(f2.Description(), 'Second folder')
        
    def testAddDuplicateFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        
        try:
            self.parent.addActionFolder('f1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        self.parent.removeActionFolder('f1')
        
        self.failIf(getattr(aq_base(self.parent), 'f1', None))
        
    def testRemoveNonExistentFolder(self):
        try:
            self.parent.removeActionFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetFolder(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        f1 = self.parent.getActionFolder('f1')
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        
    def testGetNonExistentFolder(self):
        try:
            self.parent.getActionFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllFolders(self):
        self.parent.addActionFolder('f1', 'Folder 1')
        self.parent.addActionFolder('f2', 'Folder 2')
        
        fs = self.parent.getAllActionFolders()
        ids = [f.getId() for f in fs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'f1')
        self.assertEqual(ids[1], 'f2')
        
    def testGetEmptyAllFolders(self):
        fs = self.parent.getAllActionFolders()
        self.assertEqual(len(fs), 0)
           
    #
    # Actions
    #
    
    def testAddAction(self):
        a1 = self.parent.addAction('a1', 'Action 1', 'First action', 
                                    ['cat1'], 'string:foo')
                                
        a2 = self.parent.addAction('a2', 'Action 2', 'Second action', 
                                    ['cat2', 'cat3'], 'string:bar', 
                                    ['Document', 'Image'],
                                    '/foo/bar',
                                    ['published', 'private'],
                                    ['base_edit', 'base_view'],
                                    ['Member', 'Manager'],
                                    'here/displayAction',
                                    0)
        
        self.failUnless(a1 == getattr(self.parent, 'a1'))
        self.failUnless(a2 == getattr(self.parent, 'a2'))
        
        self.assertEqual(a1.getId(), 'a1')
        self.assertEqual(a1.Title(), 'Action 1')
        self.assertEqual(a1.Description(), 'First action')
        self.assertEqual(a1.getCategories(), ('cat1',))
        self.assertEqual(a1.getTargetExpression(), 'string:foo')
        self.assertEqual(a1.getContentTypes(), ())
        self.assertEqual(a1.getMatchPath(), '')
        self.assertEqual(a1.getStates(), ())
        self.assertEqual(a1.getTemplateIds(), ())
        self.assertEqual(a1.getMatchRoles(), ())
        self.assertEqual(a1.getExpression(), '')
        self.assertEqual(a1.isActive(), 1)
        
        self.assertEqual(a2.getId(), 'a2')
        self.assertEqual(a2.Title(), 'Action 2')
        self.assertEqual(a2.Description(), 'Second action')
        self.assertEqual(a2.getCategories(), ('cat2','cat3'))
        self.assertEqual(a2.getTargetExpression(), 'string:bar')
        self.assertEqual(a2.getContentTypes(), ('Document', 'Image'))
        self.assertEqual(a2.getMatchPath(), '/foo/bar')
        self.assertEqual(a2.getStates(), ('published', 'private'))
        self.assertEqual(a2.getTemplateIds(), ('base_edit', 'base_view'))
        self.assertEqual(a2.getMatchRoles(), ('Member', 'Manager'))
        self.assertEqual(a2.getExpression(), 'here/displayAction')
        self.assertEqual(a2.isActive(), 0)
        
    def testAddDuplicateAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        
        try:
            self.parent.addAction('a1', 'Ooopsie', 'Only one action allowed', 
                                    ['cat2'], 'string:bar')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        self.parent.removeAction('a1')
        
        self.failIf(getattr(aq_base(self.parent), 'a1', None))
        
    def testRemoveNonExistentAction(self):
        try:
            self.parent.removeAction('a1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAction(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        a1 = self.parent.getAction('a1')
        
        self.assertEqual(a1.getId(), 'a1')
        self.assertEqual(a1.Title(), 'Action 1')
        self.assertEqual(a1.Description(), 'First action')
        self.assertEqual(a1.getCategories(), ('cat1',))
        self.assertEqual(a1.getTargetExpression(), 'string:foo')
        
    def testGetNonExistentAction(self):
        try:
            self.parent.getAction('a1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllActions(self):
        self.parent.addAction('a1', 'Action 1', 'First action', 
                                ['cat1'], 'string:foo')
        self.parent.addAction('a2', 'Action 2', 'Second action',
                                ['cat2'], 'string:bar')
        
        cas = self.parent.getAllActions()
        ids = [a.getId() for a in cas]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'a1')
        self.assertEqual(ids[1], 'a2')
        
    def testGetEmptyAllActions(self):
        cas = self.parent.getAllActions()
        self.assertEqual(len(cas), 0)
        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInTool))
    suite.addTest(makeSuite(TestInBundle))
    suite.addTest(makeSuite(TestInFolder))
    
    return suite

if __name__ == '__main__':
    framework()
