
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextActions.tests import catc
from Products.PloneContextActions.config import *
from Products.CMFCore.utils import getToolByName


class TestActionTypeBasics(catc.ContextActionsTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')
        self.bundle.invokeFactory('ContextAction', 'anAction')
        self.action = getattr(self.bundle, 'anAction')
        
    def testContentTypesVocab(self):

        # Since it's quite hard to keep track of all possible content
        # types here (e.g. PlonePortlets adds a whole bunch)
        # let's be a little weak and only test the default ones.
        
        defaultTypes = ['Document', 'Image', 'Link', 
                        'News Item', 'Event', 'File']
                        
        self.bundle.invokeFactory('ContextAction', 'a1')
        availableTypes = getattr(self.bundle, 'a1').getAvailableContentTypes()
        
        for t in defaultTypes:
            self.failUnless(t in availableTypes)

    def testStatesVocab(self):
        self.bundle.invokeFactory('ContextAction', 'a1')
        expectedStates = ['pending',
                          'private',
                          'published',
                          'visible']
        
        a1 = getattr(self.bundle, 'a1')
            
        actionVocab = a1.getAvailableWorkflowStates().keys()   
        actionVocab.sort()
        self.assertEqual(actionVocab, expectedStates)

    def testCategoriesVocab(self):
        self.bundle.invokeFactory('ContextAction', 'a1',
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                categories = ['cat2', 'cat3'])
                                
        a1 = getattr(self.bundle, 'a1')
        a1.setCategories(['cat1', 'cat2', 'cat4'])
        a1.reindexObject()
        
        actionVocab = a1.getExistingCategories().keys()
        actionVocab.sort()
        self.assertEqual(actionVocab, ['cat1', 'cat2', 'cat3', 'cat4'])
        
    def testRolesVocab(self):
        self.bundle.invokeFactory('ContextAction', 'a1')
        expectedRoles = ['Manager',
                         'Member',
                         'Owner',
                         'Reviewer']

        a1 = getattr(self.bundle, 'a1')
                
        actionVocab = a1.getAvailableRoles().keys()
        actionVocab.sort()
        self.assertEqual(actionVocab, expectedRoles)


class TestToolBasics(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testToolHasIndexes(self):
        self.failUnless( self.tool.indexes() )
        self.failUnless( 'match_workflow_states' in self.tool.indexes() )

    def testToolColumnsOverride(self):
        # print self.tool.columns()
        self.failUnless( self.tool.columns() )
        self.failUnless( 'Title' in self.tool.columns() )
        self.failUnless( 'getTargetExpression' in self.tool.columns() )
        self.failUnless( 'getExpression' in self.tool.columns() )

    def testActionsAreIndexed(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextAction','anAction')
        self.failUnless(self.tool.searchResults())

    def testActionsIndexGoodData(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextAction','anAction', 
                                    contentTypes=['Document'])
        self.failUnless(self.tool.searchResults())
        #print self.tool.getIndexDataForRID(int(self.tool.searchResults()[0].getRID()))
        #print self.tool.indexes()
        self.failUnless(self.tool.searchResults(match_content_types='Document'))

    def testBrainsHaveMetadata(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextAction','anAction', expression='Joe')
        self.assertEqual(self.tool.searchResults()[0].getExpression, 'Joe' )

    def testUniqueValuesFor(self):
        # Ensure uniqueValuesFor() strips MATCHALL
        # This will gain MATCHALL, since it has no contentTypes set
        self.bundle.invokeFactory('ContextAction', 'anAction')
        
        # This will not
        self.bundle.invokeFactory('ContextAction', 'anotherAction', 
                        contentTypes = 'Document')                            
        self.assertEqual(self.tool.uniqueValuesFor('match_content_types'),
                         ('Document',))

    def testEvaluateTarget(self):
        self.bundle.invokeFactory('ContextAction', 'anAction', 
                targetExpression = 'python:"%s/myScript" % \
                                     (object.absolute_url(),)')
        a = getattr(self.bundle, 'anAction')
        expected = "%s/myScript" % (self.folder.absolute_url(),)
        evaluated = a.evaluateTarget(self.folder)        
        self.assertEqual(expected, evaluated)        

    def testEvaluateExpressionMethod(self):
        self.bundle.invokeFactory('ContextAction', 'anAction', 
                targetExpression = 'python:"%s/myScript" % \
                                     (object.absolute_url(),)')
        a = getattr(self.bundle, 'anAction')
        expected = "%s/myScript" % (self.folder.absolute_url(),)
        evaluated = self.tool.evaluateExpression(a.getTargetExpression(), self.folder)        
        self.assertEqual(expected, evaluated)        

    #
    # Tests for vocab methods
    #
    
    def testCategoriesVocab(self):
        
        self.bundle.invokeFactory('ContextAction', 'a1',
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                categories = ['cat2', 'cat3'])
                                
        a1 = getattr(self.bundle, 'a1')
        a1.setCategories(['cat1', 'cat2', 'cat4'])
        a1.reindexObject()
        
        toolVocab = list(self.tool.getAvailableCategories())
        toolVocab.sort()
        self.assertEqual(toolVocab, ['cat1', 'cat2', 'cat3', 'cat4'])
        
    def testRolesVocab(self):
        self.bundle.invokeFactory('ContextAction', 'a1')
        expectedRoles = ['Manager',
                         'Member',
                         'Owner',
                         'Reviewer']
        
        a1 = getattr(self.bundle, 'a1')
                
        actionVocab = a1.getAvailableRoles().keys()
        actionVocab.sort()
        self.assertEqual(actionVocab, expectedRoles)      
    
    #
    # Tests for getActions() in the tool
    #

    def testGetActionsWithContentTypes(self):
        self.bundle.invokeFactory('ContextAction', 'TestAction')
        self.assertEqual(len(self.tool.getActions(self.folder)), 1)
        a = getattr(self.bundle, 'TestAction')
        a.setContentTypes(['Document'])
        a.reindexObject()
        self.assertEqual(len(self.tool.getActions(self.folder)), 0)
        a.setContentTypes(['Document','Folder'])
        a.reindexObject()
        self.assertEqual(len(self.tool.getActions(self.folder)), 1)
        
    def testGetActionsWithCategories(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                categories = ['cat3', 'cat4'])
        
        cat2s = self.tool.getActions(self.folder, None, ['cat2'])
        
        self.assertEqual(len(cat2s), 2)

        ids = [cat2s[0].getId(), cat2s[1].getId()]
        ids.sort()
        self.assertEqual(ids, ['a1', 'a2'])

    def testDisableCategories(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                categories = ['cat3', 'cat4'])
                                
        
        self.tool.setDisabledCategories(['cat2'])

        cat12s = self.tool.getActions(self.folder, None, ['cat1', 'cat2'])
        
        self.assertEqual(len(cat12s), 1)
        self.assertEqual(cat12s[0].getId(), 'a1')
        
    def testDisableAllCategories(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                categories = ['cat3', 'cat4'])
                                
        self.tool.setDisabledCategories(['cat1', 'cat2', 'cat3', 'cat4'])

        # Not that the categories matter...
        res = self.tool.getActions(self.folder, None, ['cat1', 'cat2'])
        
        # We don't get any, since since category is a required field
        self.assertEqual(len(res), 0)
        
    def testGetActionsWithRoles(self):
        
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                matchRoles = ['Member', 'Owner'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                matchRoles = ['Manager', 'Reviewer'])

        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextAction', 'a4')

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # User should have roles Member and Owner in context        
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 3)        
        ids = [res[0].getId(), res[1].getId(), res[2].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a1', 'a2', 'a4'])
        
    def testDisableRoles(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                matchRoles = ['Member', 'Authenticated'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                matchRoles = ['Manager', 'Reviewer'])
                                
        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextAction', 'a4')
        
        self.tool.setDisabledRoles(['Member'])

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # Match a2 (authenticated) and a4 (match all)
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId(), res[1].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a2', 'a4'])
        
    def testDisableAllRoles(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                matchRoles = ['Member', 'Authenticated'])
        self.bundle.invokeFactory('ContextAction', 'a3',
                                matchRoles = ['Manager', 'Reviewer'])
        
        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextAction', 'a4')
                
        # Regular user will have Member, Authenticated roles in context
        self.tool.setDisabledRoles(['Member', 'Authenticated'])

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # Should match a4 w/ MATCHALL, but not other ones
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0].getId(), 'a4')
        
    def testGetActionsWithTemplates(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                templateIds = ['base_view', 'base_edit'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                templateIds = ['base_edit'])
                                
        # This one has MATCHALL on template ids
        self.bundle.invokeFactory('ContextAction', 'a3')
        
        res = self.tool.getActions(self.folder, 'base_view')
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId(), res[1].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a1', 'a3'])
        
    def testGetActionsWithStates(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                states = ['published', 'visible'])
        self.bundle.invokeFactory('ContextAction', 'a2',
                                states = ['hidden'])
                                
        # This one has MATCHALL on template ids
        self.bundle.invokeFactory('ContextAction', 'a3')
        
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId(), res[1].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a1', 'a3'])
        
    def testGetActionsWithExpression(self):
        
        self.bundle.invokeFactory('ContextAction', 'a1', 
                expression = 'python:1 == 0')
        self.bundle.invokeFactory('ContextAction', 'a2',
                expression = 'python:object.getId() == "%s"' % \
                                 (self.folder.getId ()))
                                
        # This one will not evaluate expressions
        self.bundle.invokeFactory('ContextAction', 'a3')
        
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId(), res[1].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a2', 'a3'])
        
    def testGetActionsWithDisabledActions(self):
        self.bundle.invokeFactory('ContextAction', 'a1', 
                                active = 1)
        self.bundle.invokeFactory('ContextAction', 'a2',
                                active = 0)
                                
        # Default is active
        self.bundle.invokeFactory('ContextAction', 'a3')
        
        res = self.tool.getActions(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId(), res[1].getId()]
        ids.sort()
        
        self.assertEqual(ids, ['a1', 'a3'])
        
    def testGetActionsWithComplexActions(self):
        # Test slightly more real-world actions just to make sure
        # we can get there with a mixture of criteria. Also test
        # the target expression evaluation
        
        # No match due to content type, template ids
        self.bundle.invokeFactory('ContextAction', 'editDoc',
                matchRoles = ['Member'],
                templateIds = ['document_edit_form'],
                contentTypes = ['Document'],
                categories = ['ContentActions'],
                targetExpression = 'string:${object/absolute_url}/edit_more')
                                
        # Match
        self.bundle.invokeFactory('ContextAction', 'editFolder',
                matchRoles = ['Member'],
                templateIds = ['folder_edit_form'],
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['ContentActions'],
                targetExpression = 'string:${object/absolute_url}/folder_jazz')
        
        # No match due to template ids
        self.bundle.invokeFactory('ContextAction', 'viewFolder',
                matchRoles = ['Member'],
                templateIds = ['folder_listing'],
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['ContentActions'],
                targetExpression = 'string:${object/absolute_url}/folder_info')
                            
        # Match    
        self.bundle.invokeFactory('ContextAction', 'folderContents',
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['NavigationActions'],
                targetExpression = 'string:${object/absolute_url}/contents_info')
                         
        # No match due to disabled roles       
        self.bundle.invokeFactory('ContextAction', 'mailDocument',
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['MailActions'],
                targetExpression = 'string:${object/absolute_url}/mail_form')
        
        self.tool.setDisabledCategories(['MailActions'])
        
        self.setRoles(['Member'])
        
        res = self.tool.getActions (self.folder, 'folder_edit_form',
                ['ContentActions', 'NavigationActions', 'MailActions'])
        
        ids = [a.getId() for a in res]
        ids.sort()
        self.assertEqual(ids, ['editFolder', 'folderContents'])
        
        targets = [a.getTarget() for a in res]
        targets.sort()
        
        self.assertEqual(targets, 
                            ['%s/contents_info' % (self.folder.absolute_url()), 
                             '%s/folder_jazz' % (self.folder.absolute_url())])
        
        
    def testTargetExpressionCanBeEmpty(self):
        self.bundle.invokeFactory('ContextAction', 'editFolder', targetExpression = '')
        res = self.tool.getActions (self.folder)
        self.assertEqual(len(res), 1)
        from types import StringType
        self.assertEqual( type(res[0].getTarget()), StringType)
        
        
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestToolBasics))
    suite.addTest(makeSuite(TestActionTypeBasics))
    return suite

if __name__ == '__main__':
    framework()
