
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextActions.tests import catc
from Products.PloneContextActions.config import *
from Products.CMFCore.utils import getToolByName


class TestContextQuery(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testSimpleQuery(self):
        self.bundle.invokeFactory('ContextAction', 'DocumentAction')
        self.assertEqual(len(self.tool.matchByContext(self.folder)), 1)

    def testQueryForType(self):
        #make one action to only show if we are viewing a Folder
        self.bundle.invokeFactory('ContextAction', 
                                'FolderAction', 
                                contentTypes=['Folder'])
        self.assertEqual(len(self.tool.matchByContext(self.folder)), 1)
        self.assertEqual(len(self.tool.matchByContext(self.portal)), 0)

    def testQueryForRoles(self):
        #make one action to only show if we are Manager
        self.bundle.invokeFactory('ContextAction', 
                                'RoleAction', 
                                matchRoles=['Manager'])
        self.assertEqual(len(self.tool.matchByContext(self.portal)), 1)
        self.setRoles(['Member'])
        self.assertEqual(len(self.tool.matchByContext(self.portal)), 0)

    def testQueryForStates(self):
        #make one action to only show if we are in visible state
        self.workflow = self.portal.portal_workflow
        self.bundle.invokeFactory('ContextAction', 
                                'TestAction', 
                                states=['visible'])
        self.assertEqual(len(self.tool.matchByContext(self.folder)), 1)
        self.workflow.doActionFor(self.folder, 'hide')
        self.assertEqual(len(self.tool.matchByContext(self.folder)), 0)

    def testActiveField(self):
        # Make sure we only return active actions
        self.bundle.invokeFactory('ContextAction', 'ActiveAction', active = 1)
        self.bundle.invokeFactory('ContextAction', 'InactiveAction', active = 0)
        
        result = self.tool.matchByContext(self.folder)
        
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].getObject().isActive(), 1)

class TestQueryWithExpressionEval(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testNoExpression(self):
        self.bundle.invokeFactory('ContextAction', 
                                'TestAction')
        self.assertEqual(len(self.tool.match(self.folder)), 1)

    def testExpressionIsDefault(self):
        self.bundle.invokeFactory('ContextAction', 
                                'TestAction', 
                                expression='default')
        self.assertEqual(len(self.tool.match(self.folder)), 1)

    def testExpressionIsNothing(self):
        self.bundle.invokeFactory('ContextAction', 
                                'TestAction', 
                                expression='nothing')
        self.assertEqual(len(self.tool.match(self.folder)), 0)

    def testPythonExpression(self):
        self.bundle.invokeFactory('ContextAction', 
                                'TestAction', 
                                expression='python:1+1')
        self.assertEqual(len(self.tool.match(self.folder)), 1)
        self.bundle.TestAction.setExpression('python:1-1')
        self.bundle.TestAction.reindexObject()
        self.assertEqual(len(self.tool.match(self.folder)), 0)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContextQuery))
    suite.addTest(makeSuite(TestQueryWithExpressionEval))
    return suite

if __name__ == '__main__':
    framework()
