
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextActions.tests import catc
from Products.PloneContextActions.config import *
from Products.PloneContextActions.ContextAction import ContextAction
from Products.PloneContextActions.ContextCriteriaMatcher import CriterionWrapper


class TestQueries(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def addAction(self, id, **kw):
        self.bundle.invokeFactory('ContextAction', id, **kw)
        return self.bundle[id]

    def testQueryForContentTypeWithEmptyValues(self):
        self.addAction('DocumentAction', contentTypes=['Document'])
        self.addAction('NewsAction', contentTypes=['NewsItem'])
        self.addAction('AllAction')

        self.assertEqual(len(self.tool.searchResults(match_content_types=['Document'])), 2)
        self.assertEqual(len(self.tool.searchResults(match_content_types=['Document','NewsItem'])), 3)
        self.assertEqual(len(self.tool.searchResults()), 3)

    def testQueryForContentTypesMulti(self):
        self.addAction('DocumentAction', contentTypes=['Document', 'NewsItem'])
        self.addAction('NewsAction', contentTypes=['NewsItem'])

        self.assertEqual(len(self.tool.searchResults(match_content_types=['Document'])), 1)
        self.assertEqual(len(self.tool.searchResults(match_content_types=['NewsItem'])), 2)
        self.assertEqual(len(self.tool.searchResults()), 2)

    def testDictSyntaxQuery(self):
        self.addAction('DocumentAction', contentTypes=['Document'])
        self.addAction('NewsAction', contentTypes=['NewsItem'])

        self.assertEqual(len(self.tool.searchResults(
                match_content_types={'query': 'Document'})), 1)
        self.assertEqual(len(self.tool.searchResults(
                match_content_types={'query': ['Document', 'NewsItem']})), 2)

    def testDictSyntaxQueryWithEmptyValues(self):
        self.addAction('DocumentAction', contentTypes=['Document'])
        self.addAction('NewsAction', contentTypes=['NewsItem'])
        self.addAction('AllAction')

        self.assertEqual(len(self.tool.searchResults(
                match_content_types={'query': 'Document'})), 2)
        self.assertEqual(len(self.tool.searchResults(
                match_content_types={'query': ['Document', 'NewsItem']})), 3)

    def testRolesQuery(self):
        self.addAction('ManagerOnlyAction', matchRoles=['Manager'])

        self.assertEqual(len(self.tool.searchResults(match_user_roles=['Manager'])), 1)
        self.assertEqual(len(self.tool.searchResults(match_user_roles=[])), 0)
        self.assertEqual(len(self.tool.searchResults(match_user_roles=['Member'])), 0)

    def testCategoryQuery(self):
        self.addAction('FishAction', categories=['Fish'])
        self.addAction('CatAction', categories=['Cat'])

        self.assertEqual(len(self.tool.searchResults(match_categories=['Cows'])), 0)

        self.addAction('AllAction')

        self.assertEqual(len(self.tool.searchResults(match_categories=['Fish'])), 2)
        self.assertEqual(len(self.tool.searchResults(match_categories=[''])), 1)
        self.assertEqual(len(self.tool.searchResults(match_categories=['Cows'])), 1)

    def testStatesQuery(self):
        self.addAction('StateAction', states=['teststate'])
        self.assertEqual(len(self.tool.searchResults(match_workflow_states=['teststate'])), 1)
        self.assertEqual(len(self.tool.searchResults(match_workflow_states=['wrongstate'])), 0)

    def testSimplePathQuery(self):
        self.addAction('PathAction', matchPath='/site/foo')

        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/foo', 'depth': 0, 'navtree': 1})), 1)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/bar', 'depth': 0, 'navtree': 1})), 0)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': 'barf', 'depth': 0, 'navtree': 1})), 0)

    def testSubObjectPathQuery(self):
        self.addAction('PathAction', matchPath='/site/foo')

        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/foo/bar', 'depth': 0, 'navtree': 1})), 1)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/foo/bar/baz', 'depth': 0, 'navtree': 1})), 1)

    def testNonPathQuery(self):
        self.addAction('TestAction', matchPath=MATCHALL)

        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': MATCHALL, 'depth': 0, 'navtree': 1})), 1)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '', 'depth': 0, 'navtree': 1})), 1)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/bar', 'depth': 0, 'navtree': 1})), 1)

    def testAllPathQuery(self):
        self.addAction('PathAction', matchPath='/site/foo')
        self.addAction('TestAction', matchPath=MATCHALL)
        self.addAction('AllAction') # implicit MATCHALL

        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': ('/site/foo', MATCHALL), 'depth': 0, 'navtree': 1})), 3)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/foo', 'depth': 0, 'navtree': 1})), 3)
        self.assertEqual(len(self.tool.searchResults(
                match_path={'query': '/site/bar', 'depth': 0, 'navtree': 1})), 2)


class TestCriterionWrapper(catc.ContextActionsTestCase):
    '''The criterion wrapper returns (MATCHALL,) if no keyword has been specified.'''

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def addAction(self, id, **kw):
        self.bundle.invokeFactory('ContextAction', id, **kw)
        return self.bundle[id]

    def testCategories(self):
        ca = self.addAction('DogAction', categories=['Panting'])
        self.assertEqual(ca.getCategories(), ('Panting',))
        self.assertEqual(CriterionWrapper(ca).getCategories(), ('Panting',))

    def testEmptyCategories(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getCategories(), ())
        self.assertEqual(CriterionWrapper(ca).getCategories(), (MATCHALL,))

    def testContentTypes(self):
        ca = self.addAction('DogAction', contentTypes=['Document'])
        ca.setContentTypes(['Document']) # Grrr
        self.assertEqual(ca.getContentTypes(), ('Document',))
        self.assertEqual(CriterionWrapper(ca).getContentTypes(), ('Document',))

    def testEmptyContentTypes(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getContentTypes(), ())
        self.assertEqual(CriterionWrapper(ca).getContentTypes(), (MATCHALL,))

    def testPath(self):
        ca = self.addAction('DogAction', matchPath='/foo/bar/baz')
        self.assertEqual(ca.getMatchPath(), '/foo/bar/baz')
        self.assertEqual(CriterionWrapper(ca).getMatchPath(), '/foo/bar/baz')

    def testEmptyPath(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getMatchPath(), '')
        self.assertEqual(CriterionWrapper(ca).getMatchPath(), MATCHALL) # !!!

    def testStates(self):
        ca = self.addAction('DogAction', states=['published'])
        self.assertEqual(ca.getStates(), ('published',))
        self.assertEqual(CriterionWrapper(ca).getStates(), ('published',))

    def testEmptyStates(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getStates(), ())
        self.assertEqual(CriterionWrapper(ca).getStates(), (MATCHALL,))

    def testTemplateIds(self):
        ca = self.addAction('DogAction', templateIds=['document_view'])
        self.assertEqual(ca.getTemplateIds(), ('document_view',))
        self.assertEqual(CriterionWrapper(ca).getTemplateIds(), ('document_view',))

    def testEmptyTemplateIds(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getTemplateIds(), ())
        self.assertEqual(CriterionWrapper(ca).getTemplateIds(), (MATCHALL,))

    def testRoles(self):
        ca = self.addAction('DogAction', matchRoles=['Reviewer'])
        self.assertEqual(ca.getMatchRoles(), ('Reviewer',))
        self.assertEqual(CriterionWrapper(ca).getMatchRoles(), ('Reviewer',))

    def testEmptyRoles(self):
        ca = self.addAction('DogAction')
        self.assertEqual(ca.getMatchRoles(), ())
        self.assertEqual(CriterionWrapper(ca).getMatchRoles(), (MATCHALL,))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestQueries))
    suite.addTest(makeSuite(TestCriterionWrapper))
    return suite

if __name__ == '__main__':
    framework()
