
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextActions.tests import catc
from Products.PloneContextActions.config import *
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.PloneUtilities import _createObjectByType


class TestSetup(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testTypes(self):
        ids = self.portal.portal_types.objectIds()
        self.failUnless('ContextAction' in ids)
        self.failUnless('ContextActionFolder' in ids)
        self.failUnless('ContextActionBundle' in ids)
        self.failUnless('ContextActionsTool' in ids)

    def testToolInstalls(self):
        self.failUnless('portal_contextactions' in self.portal.objectIds())

    def testGettingTool(self):
        tool = getToolByName(self.portal, TOOLNAME)
        self.failUnless(tool)

    def testAddingAction(self):
        # This is superfluous now that we have tests for the management API
        self.bundle.invokeFactory('ContextAction', 'random')
        self.failUnless('random' in self.bundle.objectIds())

    def testEditingAction(self):
        self.bundle.invokeFactory('ContextAction', 'random')
        r = self.bundle.random
        r.setTitle('T')
        r.setCategories(['asdasd'])
        r.setContentTypes(['asd'])
        r.setMatchPath('sada')
        r.setStates(['sad'])
        r.setTemplateIds(['sdasdasd'])
        r.setMatchRoles(['sad'])
        r.setExpression('python:1')
        r.setActive(1)
        r.setTargetExpression('python:object.absolute_url() + "/edit"')
        r.processForm(REQUEST=self.app.REQUEST)


class TestContainment(catc.ContextActionsTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contextactions
        self.bundle = self.tool.addBundle('t1', 'Foo')
        _createObjectByType('ContextActionFolder', self.tool, id='caf')
        self.folder = self.tool.caf

    def testToolAllowedTypes(self):
        types = [x.getId() for x in self.tool.allowedContentTypes()]
        self.assertEqual(len(types), 1)
        self.failUnless('ContextActionBundle' in types)

    def testCreateBundleInTool(self):
        self.tool.invokeFactory('ContextActionBundle', id='foo')

    def testCreateDocumentInTool(self):
        # Not allowed
        self.assertRaises(ValueError, self.tool.invokeFactory, 'Document', id='foo')

    def testBundleAllowedTypes(self):
        types = [x.getId() for x in self.bundle.allowedContentTypes()]
        self.assertEqual(len(types), 2)
        self.failUnless('ContextActionFolder' in types)
        self.failUnless('ContextAction' in types)

    def testCreateFolderInBundle(self):
        self.bundle.invokeFactory('ContextActionFolder', id='foo')

    def testCreateActionInBundle(self):
        self.bundle.invokeFactory('ContextAction', id='foo')

    def testCreateDocumentInBundle(self):
        # Not allowed
        self.assertRaises(ValueError, self.bundle.invokeFactory, 'Document', id='foo')

    def testCreateFolderInFolder(self):
        self.folder.invokeFactory('ContextActionFolder', id='foo')

    def testCreateActionInFolder(self):
        self.folder.invokeFactory('ContextAction', id='foo')

    def testCreateDocumentInFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.folder.invokeFactory, 'Document', id='foo')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSetup))
    suite.addTest(makeSuite(TestContainment))
    return suite

if __name__ == '__main__':
    framework()
