from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *

from Products.CMFCore import CMFCorePermissions

from config import *
from interfaces import IContextHelpManager

class ContextHelpManager:
    """
    Mixin class to provide a cleaner API for ContextHelpBundle and 
    ContextHelpFolder to manage folders and Helps. The aim is to
    make these seems less content-ish and more like a structured class
    hierarchy.
    """
    
    __implements__ = (IContextHelpManager,)
    
    security = ClassSecurityInfo()
    
    #
    # Contained help pages management
    #
    
    security.declareProtected(CMFCorePermissions.View, 'getHelpPage')
    def getHelpPage(self, id):
        """Get the ContextHelpPage with the specified id. Raises an 
        AttributeError if the help page cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Help page %s does not exist" % (id,)
        return getattr(self, id)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllHelpPages')
    def getAllHelpPages(self):
        """Get a list of all ContextHelpPages managed directly by this 
        container
        """
        return self.objectValues(['ContextHelpPage'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                             'addHelpPage')
    def addHelpPage(self, id, title, description, body, mimetype, categories, 
                    contentTypes=None, path=None, 
                    states=None, templateIds=None, matchRoles=None,
                    expression=None, active=1, related=None):
        """Add a new ContextHelpPage. All arguments without default values are
        required. Categories should be a list, as should contentTypes,
        states, templateIds, matchRoles and related if given. Raises an 
        AttributeError if the id is already in use. Returns the newly created 
        help page.
        """
        if id in self.objectIds():
            raise AttributeError, "Help page %s already exists" % (id,)
        
        self.invokeFactory('ContextHelpPage', id)
        h = getattr(self, id)
        
        h.setTitle(title)
        h.setDescription(description)
        h.setCategories(categories)
        
        h.setBody(body, mimetype = mimetype)
        
        if contentTypes:
            h.setContentTypes(contentTypes)
        if path:
            h.setMatchPath(path)
        if states:
            h.setStates(states)
        if templateIds:
            h.setTemplateIds(templateIds)
        if matchRoles:
            h.setMatchRoles(matchRoles)
        if expression:
            h.setExpression(expression)
        
        # No condition - we want true or false!
        h.setActive(active)
            
        if related:
            h.setRelated(related)
            
        return h
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'removeHelpPage')
    def removeHelpPage(self, id):
        """Remove the given ContextHelpPage. Rasises an AttributeError if it
        cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Help page %s does not exist" % (id,)
        self.manage_delObjects([id])

    #
    # Contained Help folders management
    #
    
    security.declareProtected(CMFCorePermissions.View, 'getHelpFolder')
    def getHelpFolder(self, id):
        """Get the ContextHelpFolder with the given id. Raises an
        AttributeError if it cannot be found.
        """
        if not id in self.objectIds():
            raise AttributeError, "Help folder %s does not exist" % (id,)
        return getattr(self, id)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllHelpFolders')
    def getAllHelpFolders(self):
        """Get a list of all HelpFolders managed directly by this container.
        """
        return self.objectValues(['ContextHelpFolder'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'addHelpFolder')
    def addHelpFolder(self, id, title, description=None):
        """Add a new ContextHelpFolder. Raises an AttributeError if the id is
        already in use. Returns the newly created Help folder.
        """
        if id in self.objectIds():
            raise AttributeError, "Help folder %s already exists" % (id,)
        
        self.invokeFactory('ContextHelpFolder', id)
        folder = getattr(self, id)
        
        folder.setTitle(title)
        
        if description:
            folder.setDescription(description)
        
        return folder
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'removeHelpFolder')
    def removeHelpFolder(self, id):
        """Remove the given ContextHelpFolder. Raises an AttributeError if it
        cannot be found.
        """
        if id not in self.objectIds():
            raise AttributeError, "Help folder %s does not exist" % (id,)
        self.manage_delObjects([id])
        