from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.Archetypes.public import *

from Products.CMFCore import CMFCorePermissions

from Products.PloneContextActions.ContextCriteriaMatcher import \
        ContextCriteriaMatcher
from config import *
from interfaces import IContextHelpTool

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import createExprContext

from Acquisition import aq_base, aq_parent , aq_inner
from Acquisition import Implicit
from Globals import InitializeClass
from AccessControl import getSecurityManager

from Products.PloneContextActions.config import MATCHALL

import schemata
    
class ContextHelpTool(UniqueObject, OrderedBaseFolder, ContextCriteriaMatcher):
    """
    Defines the tool portal_contexthelp, which stores ContextHelpPages
    with context-specific help in theportal. ContextHelpPagess are stored 
    in ContextHelpBundles, which may be organised internally with 
    ContextHelpFolders. Help lookups are catalog-based, so the complexity of 
    this structure is arbitrary. 
    
    The methods of the IContextHelpTool interface should be used for 
    managing bundles, rather than the usual invokeFactory() and friends,
    for clarity and consistency.
    """
    
    __implements__ = OrderedBaseFolder.__implements__ + \
                        ContextCriteriaMatcher.__implements__ + \
                        (IContextHelpTool,)
    
    schema         = OrderedBaseFolderSchema + \
                        schemata.ContextHelpToolSchema
    security       = ClassSecurityInfo()

    portal_type    = 'ContextHelpTool' 
    meta_type      = 'ContextHelpTool' 
    archetype_name = 'Contextual help tool'
    content_icon   = 'contexthelptool.gif'
    
    allowed_content_types = ['ContextHelpBundle']
    global_allow          = 0
    filter_content_types  = 1
    allow_discussion      = 0
    
    typeDescription = "The portal_contexthelp tool manages help pages which " \
                      "are made available depending on the current context " \
                      "in the portal."
    typeDescMsgId   = 'description_edit_contexthelptool'
        
    # tool-constructors have no id argument, the id is fixed
    def __init__(self):
        OrderedBaseFolder.__init__(self,TOOLNAME)
        ContextCriteriaMatcher.__init__(self)
        self._initIndexes()

    #
    # Help queries
    #

    security.declarePublic('getHelpPages')
    def getHelp(self, context, templateId=None, categories=None, **kwargs):
        """Get the sum of help pages to be displayed for a given context. 
        You must also pass in the current template id, or None if it is not 
        relevant, and the categories to consider, or None if all categories 
        should be considered.
        
        Additional query parameters can be passed via keyword arguments.
        Returns a catalog query result set.
        """

        # Inject template id and categories into the kwargs sent to the query
        if templateId:
            kwargs['match_template_ids'] = templateId
         
        if 'match_categories' in kwargs:
            del kwargs['match_categories']   
        if categories:
            kwargs['match_categories'] = categories
            
        matches = self.match(context, **kwargs)
        return matches

    # filterContextQuery template method - declared private in base class
    def filterContextQuery(self, query):
        
        # If we are to disable any categories, get the list of categories to
        # consider (either all or the ones passed in with the categories 
        # parameter) and subtract the ones to diable.
        
        categories = query.get('match_categories', None)
        disabledCategories = self.getDisabledCategories()
        
        if disabledCategories:
        
            allCategories = []  
            if categories:
                allCategories = categories
            else:
                allCategories = self.getAvailableCategories()
            
            filteredCategories = filter(lambda x: x not in disabledCategories, 
                                          allCategories)
                                                      
            if not filteredCategories:
                query['match_categories'] = (MATCHALL,)
            else:
                query['match_categories'] = filteredCategories
        
        # Similarly, if we are to disable some roles, find the user's current
        # roles and subtract the disabled ones.
        
        disabledRoles = self.getDisabledRoles()
        
        if disabledRoles:
        
            allRoles = []
            if 'match_user_roles' in query:
                allRoles = query['match_user_roles']
            else:
                allRoles = \
                    getSecurityManager().getUser().getRolesInContext(context)
        
            filteredRoles = filter(lambda x: x not in disabledRoles, allRoles)

            query['match_user_roles'] = filteredRoles
        
    #
    # Bundle management API
    #
            
    security.declareProtected(CMFCorePermissions.View, 'getBundle')
    def getBundle(self, providerName):
        """Get the bundle for the given provider. Raises an AttributeError if
        the bundle cannot be found.
        """
        if providerName not in self.objectIds():
            raise AttributeError, "Bundle %s does not exist" % (providerName,)
        return getattr(self, providerName)
        
    security.declareProtected(CMFCorePermissions.View, 'getAllBundles')
    def getAllBundles(self):
        """Get a list of all bundles managed by the tool.
        """
        return self.objectValues(['ContextHelpBundle'])
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'addBundle')
    def addBundle(self, providerName, title, description = None):
        """Add a new Help bundle with the given title and description.
        The id of the bundle will be the same as providerName, which should
        be set to the name of the product which is providing the help bundle.
        Raises an AttributeError if a bundle for this provider already exists.
        Returns the newly created bundle.
        """
        if providerName in self.objectIds():
            raise AttributeError, "Bundle %s already exists" % (providerName,)
        
        self.invokeFactory('ContextHelpBundle', providerName)

        bundle = getattr(self, providerName)

        bundle.setTitle(title)
        
        if description:
            bundle.setDescription(description)
                            
        return bundle
        
    security.declareProtected(CMFCorePermissions.View, 'removeBundle')
    def removeBundle(self, providerName):
        """Remove the bundle managed by the given provider. Raises an
        AttributeError if it cannot be found.
        """
        if not providerName in self.objectIds():
            raise AttributeError, "Bundle %s does not exist" % (providerName,)
        self.manage_delObjects([providerName])

 
    #
    # Vocabulary methods
    #
 
    security.declarePrivate('getAvailableRoles')
    def getAvailableRoles(self):
        """
        Get a list of the roles currently in use in the portal. This helper
        method aids defining vocabularies for subclass tools which allow the 
        user to select which roles to enable.
        """
        membershipTool = getToolByName(self, 'portal_membership')
        l = DisplayList()
        for role in membershipTool.getPortalRoles():
            l.add(role, role)
        return l        
 
registerType(ContextHelpTool)

