from Interface import Base

class IContextHelpTool(Base):
    """
    Interface for context-dependent help tools. A context-dependent help 
    tool contains IContextHelpPage-implementing objects, and can find the 
    pages relevant in a given context via a call to getHelp().
    """

    def getHelp(context, tempateId, categories, **kwargs):
        """Return a catalog query search result set consisting of the
        help pages relevant to the given context, and/or the given template id.
        If the template id is not relevant, set it to None. 
        
        Return only pages in the given categories (a list). Set categories
        to None or [] if all categories should be considered.
        
        Note that the list of disabled roles and categories set on the
        tool (via the configlet or the schema mutators directly) may
        impact on the matched categories and roles.
        
        You may pass in additional parameters to match other catalog indexes.
        """
        
        pass
    
    #
    # Bundle management API
    #
                
    def getBundle(providerName):
        """Get the bundle for the given provider. Raises an AttributeError if
        the bundle cannot be found.
        """
        pass
        
    def getAllBundles():
        """Get a list of all bundles managed by the tool.
        """
        pass
   
    def addBundle(providerName, title, description=None):
        """Add a new help bundle with the given title and description.
        The id of the bundle will be the same as providerName, which should
        be set to the name of the product which is providing the help bundle.
        Raises an AttributeError if a bundle for this provider already exists.
        Returns the newly created bundle.
        """
        pass
        
    def removeBundle(providerName):
        """Remove the bundle managed by the given provider. Raises an
        AttributeError if it cannot be found.
        """
        pass    

# end of class IContextActionsTool

