# 
# PloneContextHelp schemata. Contains the schemata for the 
# ContextHelpTool, and ContextHelpPage, with their base classes and mixins.
# 

from Products.Archetypes.public import *

from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget \
    import ReferenceBrowserWidget

from Products.PloneContextActions.schemata import ContextCriteriaMatcherSchema
from Products.PloneContextActions.schemata import ContextCriteriaMixinSchema

# Put description on main edit page
ContextHelpBaseSchema = BaseSchema.copy()
ContextHelpBaseSchema['description'].isMetadata = False
ContextHelpBaseSchema['description'].schemata = 'default'

#
# ContextHelpPage schema
#

ContextHelpPageSchema = ContextHelpBaseSchema + \
                            Schema((

        StringField('title',
            accessor='Title',
            required=1,
            searchable=1,
            widget=StringWidget(description="The main title of the help page",
                        description_msgid='ContextHelp_help_title',
                        i18n_domain='ContextHelp',
                        label='Title',
                        label_msgid='ContextHelp_label_title',),
        ),

        TextField('description',
            accessor='Description',
            searchable=1,
            widget=TextAreaWidget(description="Please enter a short "          \
                        "description of the help page. This will help users "  \
                        "determine its relevancy.",
                        description_msgid='ContextHelp_help_description',
                        i18n_domain='ContextHelp',
                        label='Description',
                        label_msgid='ContextHelp_label_description',),
        ),

        TextField('body',
            required=1,
            primary=1,
            searchable=1,
            allowable_content_types=('text/html', 'text/structured',
                                        'text/plain', 'text/restructured'),
            default_content_type='text/structured',
            default_output_type='text/html',
            widget=RichWidget(description="The body text of the help page.",
                        description_msgid='ContextHelp_help_body',
                        i18n_domain='ContextHelp',
                        label='Body text',
                        label_msgid='ContextHelp_label_body',
                        rows=18),
        ),

        ReferenceField('related',
            allowed_types=('ContextHelpPage'),
            multiValued=1,
            relationship='RelatedHelpPages',
            widget=ReferenceBrowserWidget(
                description="Select any related help pages.",
                description_msgid='ContextHelp_help_related',
                i18n_domain='ContextHelp',
                label='Related',
                label_msgid='ContextHelp_label_related',
            ),
        ),

    ), marshall=RFC822Marshaller()) + ContextCriteriaMixinSchema

#
# ContextHelpTool schema
#

# Disable the id and title fields on the edit tab

ContextHelpToolSchema = ContextCriteriaMatcherSchema + Schema((

    StringField('id',
        mode='r',
        widget=StringWidget(
            visible={'edit' : 'invisible', 'view' : 'invisible'},
        ),
    ),

    ComputedField('title',
        mode='r',
        expression='"Help pages"',
        widget=ComputedWidget(
            visible={'edit' : 'invisible', 'view' : 'invisible'},
        ),
    ),

    ),)
