
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextHelp.tests import chtc
from Products.PloneContextHelp.config import *
from Products.CMFCore.utils import getToolByName

from Acquisition import aq_base

class TestInTool(chtc.ContextHelpTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contexthelp
        self.parent = self.tool

    def testAddBundle(self):
        b1 = self.parent.addBundle('b1', 'Bundle 1')
        b2 = self.parent.addBundle('b2', 'Bundle 2', 'Second bundle')
        
        self.failUnless(b1 == getattr(self.parent, 'b1'))
        self.failUnless(b2 == getattr(self.parent, 'b2'))
        
        self.assertEqual(b1.getId(), 'b1')
        self.assertEqual(b1.Title(), 'Bundle 1')
        self.assertEqual(b1.Description(), '')
        
        self.assertEqual(b2.getId(), 'b2')
        self.assertEqual(b2.Title(), 'Bundle 2')
        self.assertEqual(b2.Description(), 'Second bundle')
        
    def testAddDuplicateBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        
        try:
            self.parent.addBundle('b1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()
    
    def testRemoveBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        self.parent.removeBundle('b1')
        
        self.failIf(getattr(aq_base(self.parent), 'b1', None))
        
    def testRemoveNonExistentBundle(self):
        try:
            self.parent.removeBundle('b1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetBundle(self):
        self.parent.addBundle('b1', 'Bundle 1')
        b1 = self.parent.getBundle('b1')
        
        self.assertEqual(b1.getId(), 'b1')
        self.assertEqual(b1.Title(), 'Bundle 1')
        
    def testGetNonExistentBundle(self):
        try:
            self.parent.getBundle('b1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllBundles(self):
        self.parent.addBundle('b1', 'Bundle 1')
        self.parent.addBundle('b2', 'Bundle 2')
        
        bs = self.parent.getAllBundles()
        ids = [b.getId() for b in bs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'b1')
        self.assertEqual(ids[1], 'b2')
        
    def testGetEmptyAllBundles(self):
        bs = self.parent.getAllBundles()
        self.assertEqual(len(bs), 0)

class TestInBundle(chtc.ContextHelpTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contexthelp
        self.bundle = self.tool.addBundle('b1', 'Bundle 1')
        self.parent = self.bundle
    
    #
    # Help folders
    #
    
    def testAddFolder(self):
        f1 = self.parent.addHelpFolder('f1', 'Folder 1')
        f2 = self.parent.addHelpFolder('f2', 'Folder 2', 'Second folder')
        
        self.failUnless(f1 == getattr(self.parent, 'f1'))
        self.failUnless(f2 == getattr(self.parent, 'f2'))
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        self.assertEqual(f1.Description(), '')
        
        self.assertEqual(f2.getId(), 'f2')
        self.assertEqual(f2.Title(), 'Folder 2')
        self.assertEqual(f2.Description(), 'Second folder')
        
    def testAddDuplicateFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        
        try:
            self.parent.addHelpFolder('f1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        self.parent.removeHelpFolder('f1')
        
        self.failIf(getattr(aq_base(self.parent), 'f1', None))
        
    def testRemoveNonExistentFolder(self):
        try:
            self.parent.removeHelpFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        f1 = self.parent.getHelpFolder('f1')
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        
    def testGetNonExistentFolder(self):
        try:
            self.parent.getHelpFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllFolders(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        self.parent.addHelpFolder('f2', 'Folder 2')
        
        fs = self.parent.getAllHelpFolders()
        ids = [f.getId() for f in fs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'f1')
        self.assertEqual(ids[1], 'f2')
        
    def testGetEmptyAllFolders(self):
        fs = self.parent.getAllHelpFolders()
        self.assertEqual(len(fs), 0)
        
    #
    # Help
    #
    
    def testAddHelpPage(self):
        h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                     'Help goes here', 'text/plain',
                                     ['cat1'])
                                
        h2 = self.parent.addHelpPage('h2', 'Help 2', 'Second help page', 
                                     'Help also goes here', 'text/html',
                                     ['cat2', 'cat3'], 
                                     ['Document', 'Image'],
                                     '/foo/bar',
                                     ['published', 'private'],
                                     ['base_edit', 'base_view'],
                                     ['Member', 'Manager'],
                                     'here/displayHelp',
                                     0,
                                     (h1.UID()))
        
        self.failUnless(h1 == getattr(self.parent, 'h1'))
        self.failUnless(h2 == getattr(self.parent, 'h2'))
        
        self.assertEqual(h1.getId(), 'h1')
        self.assertEqual(h1.Title(), 'Help 1')
        self.assertEqual(h1.Description(), 'First help page')
        self.assertEqual(h1.getBody(), '<p>Help goes here</p>')
        self.assertEqual(h1.getField('body').getContentType(h1), 'text/plain')
        self.assertEqual(h1.getCategories(), ('cat1',))
        self.assertEqual(h1.getContentTypes(), ())
        self.assertEqual(h1.getMatchPath(), '')
        self.assertEqual(h1.getStates(), ())
        self.assertEqual(h1.getTemplateIds(), ())
        self.assertEqual(h1.getMatchRoles(), ())
        self.assertEqual(h1.getExpression(), '')
        self.assertEqual(h1.isActive(), 1)
        self.assertEqual(h1.getRelated(), [])
        
        self.assertEqual(h2.getId(), 'h2')
        self.assertEqual(h2.Title(), 'Help 2')
        self.assertEqual(h2.Description(), 'Second help page')
        self.assertEqual(h2.getBody(), 'Help also goes here')
        self.assertEqual(h2.getField('body').getContentType(h2), 'text/html')
        self.assertEqual(h2.getCategories(), ('cat2','cat3'))
        self.assertEqual(h2.getContentTypes(), ('Document', 'Image'))
        self.assertEqual(h2.getMatchPath(), '/foo/bar')
        self.assertEqual(h2.getStates(), ('published', 'private'))
        self.assertEqual(h2.getTemplateIds(), ('base_edit', 'base_view'))
        self.assertEqual(h2.getMatchRoles(), ('Member', 'Manager'))
        self.assertEqual(h2.getExpression(), 'here/displayHelp')
        self.assertEqual(h2.isActive(), 0)
        self.assertEqual(h2.getRelated(), [h1])
        
    def testAddDuplicateHelpPage(self):
        h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                     'Help goes here', 'text/plain',
                                     ['cat1'])
        
        try:
            h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                         'Help goes here', 'text/plain',
                                         ['cat1'])
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveHelpPage(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
                                    
        self.parent.removeHelpPage('h1')
        
        self.failIf(getattr(aq_base(self.parent), 'h1', None))
        
    def testRemoveNonExistentHelpPage(self):
        try:
            self.parent.removeHelpPage('h1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetHelpPage(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
        h1 = self.parent.getHelpPage('h1')
        
        self.assertEqual(h1.getId(), 'h1')
        self.assertEqual(h1.Title(), 'Help 1')
        self.assertEqual(h1.Description(), 'First help page')
        self.assertEqual(h1.getBody(), '<p>Help goes here</p>')
        self.assertEqual(h1.getCategories(), ('cat1',))
        
    def testGetNonExistentHelpPage(self):
        try:
            self.parent.getHelp('h1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllHelpPages(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
        self.parent.addHelpPage('h2', 'Help 2', 'Second help page', 
                                    'More help goes here', 'text/html',
                                    ['cat2'])
        
        cas = self.parent.getAllHelpPages()
        ids = [a.getId() for a in cas]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'h1')
        self.assertEqual(ids[1], 'h2')
        
    def testGetEmptyAllHelpPages(self):
        hps = self.parent.getAllHelpPages()
        self.assertEqual(len(hps), 0)
        
class TestInFolder(chtc.ContextHelpTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        
        self.tool   = self.portal.portal_contexthelp
        self.bundle = self.tool.addBundle('b1', 'Bundle 1')
        self.folder = self.bundle.addHelpFolder('f1', 'Folder 1')
        self.parent = self.folder

    #
    # Help folders
    #

    def testAddFolder(self):
        f1 = self.parent.addHelpFolder('f1', 'Folder 1')
        f2 = self.parent.addHelpFolder('f2', 'Folder 2', 'Second folder')
        
        self.failUnless(f1 == getattr(self.parent, 'f1'))
        self.failUnless(f2 == getattr(self.parent, 'f2'))
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        self.assertEqual(f1.Description(), '')
        
        self.assertEqual(f2.getId(), 'f2')
        self.assertEqual(f2.Title(), 'Folder 2')
        self.assertEqual(f2.Description(), 'Second folder')
        
    def testAddDuplicateFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        
        try:
            self.parent.addHelpFolder('f1', 'Ooopsie')
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        self.parent.removeHelpFolder('f1')
        
        self.failIf(getattr(aq_base(self.parent), 'f1', None))
        
    def testRemoveNonExistentFolder(self):
        try:
            self.parent.removeHelpFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetFolder(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        f1 = self.parent.getHelpFolder('f1')
        
        self.assertEqual(f1.getId(), 'f1')
        self.assertEqual(f1.Title(), 'Folder 1')
        
    def testGetNonExistentFolder(self):
        try:
            self.parent.getHelpFolder('f1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllFolders(self):
        self.parent.addHelpFolder('f1', 'Folder 1')
        self.parent.addHelpFolder('f2', 'Folder 2')
        
        fs = self.parent.getAllHelpFolders()
        ids = [f.getId() for f in fs]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'f1')
        self.assertEqual(ids[1], 'f2')
        
    def testGetEmptyAllFolders(self):
        fs = self.parent.getAllHelpFolders()
        self.assertEqual(len(fs), 0)
           
    #
    # Help pages
    #
    
    def testAddHelpPage(self):
        h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                     'Help goes here', 'text/plain',
                                     ['cat1'])
                                
        h2 = self.parent.addHelpPage('h2', 'Help 2', 'Second help page', 
                                     'Help also goes here', 'text/html',
                                     ['cat2', 'cat3'], 
                                     ['Document', 'Image'],
                                     '/foo/bar',
                                     ['published', 'private'],
                                     ['base_edit', 'base_view'],
                                     ['Member', 'Manager'],
                                     'here/displayHelp',
                                     0)
        
        self.failUnless(h1 == getattr(self.parent, 'h1'))
        self.failUnless(h2 == getattr(self.parent, 'h2'))
        
        self.assertEqual(h1.getId(), 'h1')
        self.assertEqual(h1.Title(), 'Help 1')
        self.assertEqual(h1.Description(), 'First help page')
        self.assertEqual(h1.getBody(), '<p>Help goes here</p>')
        self.assertEqual(h1.getField('body').getContentType(h1), 'text/plain')
        self.assertEqual(h1.getCategories(), ('cat1',))
        self.assertEqual(h1.getContentTypes(), ())
        self.assertEqual(h1.getMatchPath(), '')
        self.assertEqual(h1.getStates(), ())
        self.assertEqual(h1.getTemplateIds(), ())
        self.assertEqual(h1.getMatchRoles(), ())
        self.assertEqual(h1.getExpression(), '')
        self.assertEqual(h1.isActive(), 1)
        
        self.assertEqual(h2.getId(), 'h2')
        self.assertEqual(h2.Title(), 'Help 2')
        self.assertEqual(h2.Description(), 'Second help page')
        self.assertEqual(h2.getBody(), 'Help also goes here')
        self.assertEqual(h2.getField('body').getContentType(h2), 'text/html')
        self.assertEqual(h2.getCategories(), ('cat2','cat3'))
        self.assertEqual(h2.getContentTypes(), ('Document', 'Image'))
        self.assertEqual(h2.getMatchPath(), '/foo/bar')
        self.assertEqual(h2.getStates(), ('published', 'private'))
        self.assertEqual(h2.getTemplateIds(), ('base_edit', 'base_view'))
        self.assertEqual(h2.getMatchRoles(), ('Member', 'Manager'))
        self.assertEqual(h2.getExpression(), 'here/displayHelp')
        self.assertEqual(h2.isActive(), 0)
        
    def testAddDuplicateHelpPage(self):
        h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                     'Help goes here', 'text/plain',
                                     ['cat1'])
        
        try:
            h1 = self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                         'Help goes here', 'text/plain',
                                         ['cat1'])
        except AttributeError:
            pass
        else:
            self.fail()

    def testRemoveHelpPage(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
                                    
        self.parent.removeHelpPage('h1')
        
        self.failIf(getattr(aq_base(self.parent), 'h1', None))
        
    def testRemoveNonExistentHelpPage(self):
        try:
            self.parent.removeHelpPage('h1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetHelpPage(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
        h1 = self.parent.getHelpPage('h1')
        
        self.assertEqual(h1.getId(), 'h1')
        self.assertEqual(h1.Title(), 'Help 1')
        self.assertEqual(h1.Description(), 'First help page')
        self.assertEqual(h1.getBody(), '<p>Help goes here</p>')
        self.assertEqual(h1.getCategories(), ('cat1',))
        
    def testGetNonExistentHelpPage(self):
        try:
            self.parent.getHelp('h1')
        except AttributeError:
            pass
        else:
            self.fail()
        
    def testGetAllHelpPages(self):
        self.parent.addHelpPage('h1', 'Help 1', 'First help page', 
                                    'Help goes here', 'text/plain',
                                    ['cat1'])
        self.parent.addHelpPage('h2', 'Help 2', 'Second help page', 
                                    'More help goes here', 'text/html',
                                    ['cat2'])
        
        cas = self.parent.getAllHelpPages()
        ids = [a.getId() for a in cas]
        
        self.assertEqual(len(ids), 2)
        
        ids.sort ()
        self.assertEqual(ids[0], 'h1')
        self.assertEqual(ids[1], 'h2')
        
    def testGetEmptyAllHelpPages(self):
        hps = self.parent.getAllHelpPages()
        self.assertEqual(len(hps), 0)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInTool))
    suite.addTest(makeSuite(TestInBundle))
    suite.addTest(makeSuite(TestInFolder))
    
    return suite

if __name__ == '__main__':
    framework()
