
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase

from Products.PloneContextHelp.tests import chtc
from Products.PloneContextHelp.config import *
from Products.CMFCore.utils import getToolByName

from Products.PloneContextActions.config import MATCHALL

class TestSetup(chtc.ContextHelpTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contexthelp
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testToolInstalls(self):
        self.failUnless('portal_contexthelp' in self.portal.objectIds())

    def testGettingTool(self):
        tool = getToolByName(self.portal, TOOLNAME)
        self.failUnless(tool)

    def testAddingHelp(self):
        # This is superfluous now that we have tests for the management API
        self.bundle.invokeFactory('ContextHelpPage', 'random')
        self.failUnless('random' in self.bundle.objectIds())

    def testEditingHelp(self):
        self.bundle.invokeFactory('ContextHelpPage', 'random')
        r = self.bundle.random
        r.setTitle('T')
        r.setCategories(['asdasd'])
        r.setContentTypes(['asd'])
        r.setMatchPath('sada')
        r.setStates(['sad'])
        r.setTemplateIds(['sdasdasd'])
        r.setMatchRoles(['sad'])
        r.setExpression('python:1')
        r.setActive(1)
        r.setBody('some body text', mimetype='text/structured')
        r.processForm(REQUEST=self.app.REQUEST)

class TestHelpTypeBasics(chtc.ContextHelpTestCase):
    
    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contexthelp
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')
        
    def testContentTypesVocab(self):

        # Since it's quite hard to keep track of all possible content
        # types here (e.g. PlonePortlets adds a whole bunch)
        # let's be a little weak and only test the default ones.
        
        defaultTypes = ['Document', 'Image', 'Link', 
                        'News Item', 'Event', 'File']
                        
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        availableTypes = getattr(self.bundle, 'h1').getAvailableContentTypes()
        
        for t in defaultTypes:
            self.failUnless(t in availableTypes)

    def testStatesVocab(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        expectedStates = ['pending',
                          'private',
                          'published',
                          'visible']
        
        h1 = getattr(self.bundle, 'h1')
            
        helpVocab = h1.getAvailableWorkflowStates().keys()   
        helpVocab.sort()
        self.assertEqual(helpVocab, expectedStates)

    def testCategoriesVocab(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1',
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                categories = ['cat2', 'cat3'])
                                
        h1 = getattr(self.bundle, 'h1')
        h1.setCategories(['cat1', 'cat2', 'cat4'])
        h1.reindexObject()
        
        helpVocab = h1.getExistingCategories().keys()
        helpVocab.sort()
        self.assertEqual(helpVocab, ['cat1', 'cat2', 'cat3', 'cat4'])
        
    def testRolesVocab(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        expectedRoles = ['Manager',
                         'Member',
                         'Owner',
                         'Reviewer']

        h1 = getattr(self.bundle, 'h1')
                
        helpVocab = h1.getAvailableRoles().keys()
        helpVocab.sort()
        self.assertEqual(helpVocab, expectedRoles)

class TestToolBasics(chtc.ContextHelpTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.tool = self.portal.portal_contexthelp
        self.bundle = self.tool.addBundle('t1', 'Test bundle 1')

    def testToolHasIndexes(self):
        self.failUnless(self.tool.indexes())
        self.failUnless('match_workflow_states' in self.tool.indexes())

    def testToolHasColumns(self):
        self.failUnless(self.tool.columns())
        self.failUnless('Title' in self.tool.columns())

    def testHelpPagesAreIndexed(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextHelpPage','h1')
        self.failUnless(self.tool.searchResults())

    def testHelpPagesIndexGoodData(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextHelpPage','h1',
                                    contentTypes=['Document'])
        self.failUnless(self.tool.searchResults())
        self.failUnless(self.tool.searchResults(match_content_types='Document'))

    def testBrainsHaveMetadata(self):
        self.failIf(self.tool.searchResults())
        self.bundle.invokeFactory('ContextHelpPage','h1', expression='Joe')
        self.assertEqual(self.tool.searchResults()[0].getExpression, 'Joe' )

    def testUniqueValuesFor(self):
        # Ensure uniqueValuesFor() strips MATCHALL
        # This will gain MATCHALL, since it has no contentTypes set
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        
        # This will not
        self.bundle.invokeFactory('ContextHelpPage', 'h2', 
                        contentTypes = 'Document')                            
        self.assertEqual(self.tool.uniqueValuesFor('match_content_types'),
                         ('Document',))

    #
    # Tests for vocab methods
    #
    
    def testCategoriesVocab(self):
        
        self.bundle.invokeFactory('ContextHelpPage', 'h1',
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                categories = ['cat2', 'cat3'])
                                
        h1 = getattr(self.bundle, 'h1')
        h1.setCategories(['cat1', 'cat2', 'cat4'])
        h1.reindexObject()
        
        toolVocab = list(self.tool.getAvailableCategories())
        toolVocab.sort()
        self.assertEqual(toolVocab, ['cat1', 'cat2', 'cat3', 'cat4'])
        
    def testRolesVocab(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        expectedRoles = ['Manager',
                         'Member',
                         'Owner',
                         'Reviewer']
        
        h1 = getattr(self.bundle, 'h1')
                
        HelpVocab = h1.getAvailableRoles().keys()
        HelpVocab.sort()
        self.assertEqual(HelpVocab, expectedRoles)      
    
    #
    # Tests for getHelp() in the tool
    #

    def testGetHelpWithContentTypes(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1')
        self.assertEqual(len(self.tool.getHelp(self.folder)), 1)
        a = getattr(self.bundle, 'h1')
        a.setContentTypes(['Document'])
        a.reindexObject()
        self.assertEqual(len(self.tool.getHelp(self.folder)), 0)
        a.setContentTypes(['Document','Folder'])
        a.reindexObject()
        self.assertEqual(len(self.tool.getHelp(self.folder)), 1)
        
    def testGetHelpWithCategories(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                categories = ['cat3', 'cat4'])
        
        cat2s = self.tool.getHelp(self.folder, None, ['cat2'])
        
        self.assertEqual(len(cat2s), 2)

        ids = [cat2s[0].getId, cat2s[1].getId]
        ids.sort()
        self.assertEqual(ids, ['h1', 'h2'])

    def testDisableCategories(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                categories = ['cat3', 'cat4'])
                                
        
        self.tool.setDisabledCategories(['cat2'])

        cat12s = self.tool.getHelp(self.folder, None, ['cat1', 'cat2'])
        
        self.assertEqual(len(cat12s), 1)
        self.assertEqual(cat12s[0].getId, 'h1')
        
    def testDisableAllCategories(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                categories = ['cat1', 'cat2'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                categories = ['cat2', 'cat3'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                categories = ['cat3', 'cat4'])
                                
        self.tool.setDisabledCategories(['cat1', 'cat2', 'cat3', 'cat4'])

        # Not that the categories matter...
        res = self.tool.getHelp(self.folder, None, ['cat1', 'cat2'])
        
        # We don't get any, since since category is a required field
        self.assertEqual(len(res), 0)
        
    def testGetHelpWithRoles(self):
        
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                matchRoles = ['Member', 'Owner'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                matchRoles = ['Manager', 'Reviewer'])

        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextHelpPage', 'h4')

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # User should have roles Member and Owner in context        
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 3)        
        ids = [res[0].getId, res[1].getId, res[2].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h1', 'h2', 'h4'])
        
    def testDisableRoles(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                matchRoles = ['Member', 'Authenticated'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                matchRoles = ['Manager', 'Reviewer'])
                                
        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextHelpPage', 'h4')
        
        self.tool.setDisabledRoles(['Member'])

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # Match h2 (authenticated) and h4 (match all)
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId, res[1].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h2', 'h4'])
        
    def testDisableAllRoles(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                matchRoles = ['Manager', 'Member'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                matchRoles = ['Member', 'Authenticated'])
        self.bundle.invokeFactory('ContextHelpPage', 'h3',
                                matchRoles = ['Manager', 'Reviewer'])
        
        # This one has MATCHALL on roles
        self.bundle.invokeFactory('ContextHelpPage', 'h4')
                
        # Regular user will have Member, Authenticated roles in context
        self.tool.setDisabledRoles(['Member', 'Authenticated'])

        # Make us Member so we can test the more common scenario
        self.setRoles(['Member'])

        # Should match h4 w/ MATCHALL, but not other ones
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0].getId, 'h4')
        
    def testGetHelpWithTemplates(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                templateIds = ['base_view', 'base_edit'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                templateIds = ['base_edit'])
                                
        # This one has MATCHALL on template ids
        self.bundle.invokeFactory('ContextHelpPage', 'h3')
        
        res = self.tool.getHelp(self.folder, 'base_view')
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId, res[1].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h1', 'h3'])
        
    def testGetHelpWithStates(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                states = ['published', 'visible'])
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                states = ['hidden'])
                                
        # This one has MATCHALL on template ids
        self.bundle.invokeFactory('ContextHelpPage', 'h3')
        
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId, res[1].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h1', 'h3'])
        
    def testGetHelpWithExpression(self):
        
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                expression = 'python:1 == 0')
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                expression = 'python:object.getId() == "%s"' % \
                                 (self.folder.getId ()))
                                
        # This one will not evaluate expressions
        self.bundle.invokeFactory('ContextHelpPage', 'h3')
        
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId, res[1].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h2', 'h3'])
        
    def testGetHelpWithDisabledPages(self):
        self.bundle.invokeFactory('ContextHelpPage', 'h1', 
                                active = 1)
        self.bundle.invokeFactory('ContextHelpPage', 'h2',
                                active = 0)
                                
        # Default is active
        self.bundle.invokeFactory('ContextHelpPage', 'h3')
        
        res = self.tool.getHelp(self.folder)
        
        self.assertEqual(len(res), 2)
        ids = [res[0].getId, res[1].getId]
        ids.sort()
        
        self.assertEqual(ids, ['h1', 'h3'])
        
    def testGetHelpWithComplexPPages(self):
        # Test slightly more real-world help pages just to make sure
        # we can get there with a mixture of criteria.
            
        # No match due to content type, template ids
        self.bundle.invokeFactory('ContextHelpPage', 'editDoc',
                matchRoles = ['Member'],
                templateIds = ['document_edit_form'],
                contentTypes = ['Document'],
                categories = ['ContentHelp'],
                body = 'this is a test')
                                
        # Match
        self.bundle.invokeFactory('ContextHelpPage', 'editFolder',
                matchRoles = ['Member'],
                templateIds = ['folder_edit_form'],
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['ContentHelp'],
                body = 'another test')
        
        # No match due to template ids
        self.bundle.invokeFactory('ContextHelpPage', 'viewFolder',
                matchRoles = ['Member'],
                templateIds = ['folder_listing'],
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['ContentHelp'],
                body = 'strangely, a test')
                            
        # Match    
        self.bundle.invokeFactory('ContextHelpPage', 'folderContents',
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['NavigationHelp'],
                body = 'test this, you fool')
                         
        # No match due to disabled roles       
        self.bundle.invokeFactory('ContextHelpPage', 'mailDocument',
                contentTypes = ['Folder', 'ATFolder'],
                categories = ['MailHelp'],
                body = 'test anything but this')
        
        self.tool.setDisabledCategories(['MailHelp'])
        
        self.setRoles(['Member'])
        
        res = self.tool.getHelp (self.folder, 'folder_edit_form',
                ['ContentHelp', 'NavigationHelp', 'MailHelp'])
        
        ids = [a.getId for a in res]
        ids.sort()
        self.assertEqual(ids, ['editFolder', 'folderContents'])
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSetup))
    suite.addTest(makeSuite(TestToolBasics))
    suite.addTest(makeSuite(TestHelpTypeBasics))
    
    
    return suite

if __name__ == '__main__':
    framework()
